@@ -56,23 +56,26 @@ def _on_soap_modified():
 	"""Always relates to the active patient."""
 	gmHooks.run_hook_script(hook = u'after_soap_modified')
 
-gmDispatcher.connect(_on_soap_modified, u'clin_narrative_mod_db')
+gmDispatcher.connect(_on_soap_modified, u'clin.clin_narrative_mod_db')
 
 #============================================================
 class cNarrative(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one clinical free text entry.
 	"""
-	_cmd_fetch_payload = u"select *, xmin_clin_narrative from clin.v_pat_narrative where pk_narrative = %s"
+	_cmd_fetch_payload = u"SELECT * FROM clin.v_narrative WHERE pk_narrative = %s"
 	_cmds_store_payload = [
 		u"""update clin.clin_narrative set
 				narrative = %(narrative)s,
 				clin_when = %(date)s,
 				soap_cat = lower(%(soap_cat)s),
-				fk_encounter = %(pk_encounter)s
-			where
-				pk=%(pk_narrative)s and
-				xmin=%(xmin_clin_narrative)s""",
-		u"""select xmin_clin_narrative from clin.v_pat_narrative where pk_narrative=%(pk_narrative)s"""
+				fk_encounter = %(pk_encounter)s,
+				fk_episode = %(pk_episode)s
+			WHERE
+				pk = %(pk_narrative)s
+					AND
+				xmin = %(xmin_clin_narrative)s
+			RETURNING
+				xmin AS xmin_clin_narrative"""
 		]
 
 	_updatable_fields = [
@@ -92,7 +95,7 @@ class cNarrative(gmBusinessDBObject.cBusinessDBObject):
 				text = _('%s: %s by %.8s (v%s)\n%s') % (
 					self._payload[self._idx['date']].strftime('%x %H:%M'),
 					soap_cat2l10n_str[self._payload[self._idx['soap_cat']]],
-					self._payload[self._idx['provider']],
+					self._payload[self._idx['modified_by']],
 					self._payload[self._idx['row_version']],
 					self._payload[self._idx['narrative']]
 				),
@@ -105,7 +108,7 @@ class cNarrative(gmBusinessDBObject.cBusinessDBObject):
 				self._payload[self._idx['date']].strftime('%x %H:%M'),
 				soap_cat2l10n[self._payload[self._idx['soap_cat']]],
 				self._payload[self._idx['narrative']],
-				self._payload[self._idx['provider']]
+				self._payload[self._idx['modified_by']]
 			)
 			if len(txt) > width:
 				txt = txt[:width] + gmTools.u_ellipsis
@@ -202,7 +205,7 @@ def search_text_across_emrs(search_term=None):
 
 	return rows
 #============================================================
-def create_clin_narrative(narrative=None, soap_cat=None, episode_id=None, encounter_id=None):
+def create_clin_narrative(narrative=None, soap_cat=None, episode_id=None, encounter_id=None, link_obj=None):
 	"""Creates a new clinical narrative entry
 
 		narrative - free text clinical narrative
@@ -223,9 +226,7 @@ def create_clin_narrative(narrative=None, soap_cat=None, episode_id=None, encoun
 	# FIXME: this should check for .provider = current_user but
 	# FIXME: the view has provider mapped to their staff alias
 	cmd = u"""
-		SELECT
-			*, xmin_clin_narrative
-		FROM clin.v_pat_narrative
+		SELECT * FROM clin.v_narrative
 		WHERE
 			pk_encounter = %(enc)s
 				AND
@@ -241,7 +242,7 @@ def create_clin_narrative(narrative=None, soap_cat=None, episode_id=None, encoun
 		'soap': soap_cat,
 		'narr': narrative
 	}
-	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+	rows, idx = gmPG2.run_ro_queries(link_obj = link_obj, queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 	if len(rows) == 1:
 		narrative = cNarrative(row = {'pk_field': 'pk_narrative', 'data': rows[0], 'idx': idx})
 		return (True, narrative)
@@ -256,13 +257,12 @@ def create_clin_narrative(narrative=None, soap_cat=None, episode_id=None, encoun
 		 'args': [encounter_id, episode_id, narrative, soap_cat]
 		},
 		{'cmd': u"""
-			SELECT *, xmin_clin_narrative
-			FROM clin.v_pat_narrative
+			SELECT * FROM clin.v_narrative
 			WHERE
 				pk_narrative = currval(pg_get_serial_sequence('clin.clin_narrative', 'pk'))"""
 		}
 	]
-	rows, idx = gmPG2.run_rw_queries(queries = queries, return_data = True, get_col_idx = True)
+	rows, idx = gmPG2.run_rw_queries(link_obj = link_obj, queries = queries, return_data = True, get_col_idx = True)
 
 	narrative = cNarrative(row = {'pk_field': 'pk_narrative', 'idx': idx, 'data': rows[0]})
 	return (True, narrative)
@@ -309,7 +309,7 @@ def get_narrative(since=None, until=None, encounters=None, episodes=None, issues
 		args['pat'] = patient
 
 	if soap_cats is not None:
-		where_parts.append(u'soap_cat IN %(soap_cats)s')
+		where_parts.append(u'c_vn.soap_cat IN %(soap_cats)s')
 		args['soap_cats'] = tuple(soap_cats)
 
 	if order_by is None:
@@ -319,11 +319,11 @@ def get_narrative(since=None, until=None, encounters=None, episodes=None, issues
 
 	cmd = u"""
 		SELECT
-			cvpn.*,
-			(SELECT rank FROM clin.soap_cat_ranks WHERE soap_cat = cvpn.soap_cat)
-				AS soap_rank
+			c_vn.*,
+			c_scr.rank AS soap_rank
 		FROM
-			clin.v_pat_narrative cvpn
+			clin.v_narrative c_vn
+				LEFT JOIN clin.soap_cat_ranks c_scr ON c_vn.soap_cat = c_scr.soap_cat
 		WHERE
 			%s
 		%s
@@ -343,7 +343,7 @@ def get_narrative(since=None, until=None, encounters=None, episodes=None, issues
 		filtered_narrative = filter(lambda narr: narr['date'] < until, filtered_narrative)
 
 	if providers is not None:
-		filtered_narrative = filter(lambda narr: narr['provider'] in providers, filtered_narrative)
+		filtered_narrative = filter(lambda narr: narr['modified_by'] in providers, filtered_narrative)
 
 	return filtered_narrative
 
