@@ -1,4 +1,4 @@
-# -*- coding: latin-1 -*-
+# -*- coding: utf-8 -*-
 """GNUmed forms classes
 
 Business layer for printing all manners of forms, letters, scripts etc.
@@ -9,7 +9,11 @@ license: GPL v2 or later
 __author__ ="Ian Haywood <ihaywood@gnu.org>, karsten.hilbert@gmx.net"
 
 
-import os, sys, time, os.path, logging
+import os
+import sys
+import time
+import os.path
+import logging
 import codecs
 import re as regex
 import shutil
@@ -42,7 +46,7 @@ from Gnumed.pycommon import gmPG2
 from Gnumed.business import gmPerson
 from Gnumed.business import gmStaff
 from Gnumed.business import gmPersonSearch
-from Gnumed.business import gmSurgery
+from Gnumed.business import gmPraxis
 
 
 _log = logging.getLogger('gm.forms')
@@ -125,28 +129,31 @@ class cFormTemplateType_MatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 			ORDER BY list_label
 		"""
 		gmMatchProvider.cMatchProvider_SQL2.__init__(self, queries = [query])
+
 #============================================================
 class cFormTemplate(gmBusinessDBObject.cBusinessDBObject):
 
-	_cmd_fetch_payload = u'select * from ref.v_paperwork_templates where pk_paperwork_template = %s'
+	_cmd_fetch_payload = u'SELECT * FROM ref.v_paperwork_templates WHERE pk_paperwork_template = %s'
 
 	_cmds_store_payload = [
-		u"""update ref.paperwork_templates set
+		u"""UPDATE ref.paperwork_templates SET
 				name_short = %(name_short)s,
 				name_long = %(name_long)s,
 				fk_template_type = %(pk_template_type)s,
 				instance_type = %(instance_type)s,
 				engine = %(engine)s,
 				in_use = %(in_use)s,
+				edit_after_substitution = %(edit_after_substitution)s,
 				filename = %(filename)s,
 				external_version = %(external_version)s
-			where
-				pk = %(pk_paperwork_template)s and
+			WHERE
+				pk = %(pk_paperwork_template)s
+					AND
 				xmin = %(xmin_paperwork_template)s
-		""",
-		u"""select xmin_paperwork_template from ref.v_paperwork_templates where pk_paperwork_template = %(pk_paperwork_template)s"""
+			RETURNING
+				xmin AS xmin_paperwork_template
+		"""
 	]
-
 	_updatable_fields = [
 		u'name_short',
 		u'name_long',
@@ -155,7 +162,8 @@ class cFormTemplate(gmBusinessDBObject.cBusinessDBObject):
 		u'instance_type',
 		u'engine',
 		u'in_use',
-		u'filename'
+		u'filename',
+		u'edit_after_substitution'
 	]
 
 	_suffix4engine = {
@@ -235,6 +243,7 @@ class cFormTemplate(gmBusinessDBObject.cBusinessDBObject):
 		form = engine(template_file = fname)
 		form.template = self
 		return form
+
 #============================================================
 def get_form_template(name_long=None, external_version=None):
 	cmd = u'select pk from ref.paperwork_templates where name_long = %(lname)s and external_version = %(ver)s'
@@ -246,6 +255,7 @@ def get_form_template(name_long=None, external_version=None):
 		return None
 
 	return cFormTemplate(aPK_obj = rows[0]['pk'])
+
 #------------------------------------------------------------
 def get_form_templates(engine=None, active_only=False, template_types=None, excluded_types=None):
 	"""Load form templates."""
@@ -276,27 +286,38 @@ def get_form_templates(engine=None, active_only=False, template_types=None, excl
 	templates = [ cFormTemplate(row = {'pk_field': 'pk_paperwork_template', 'data': r, 'idx': idx}) for r in rows ]
 
 	return templates
+
 #------------------------------------------------------------
 def create_form_template(template_type=None, name_short=None, name_long=None):
-
-	cmd = u'insert into ref.paperwork_templates (fk_template_type, name_short, name_long, external_version) values (%(type)s, %(nshort)s, %(nlong)s, %(ext_version)s)'
-	rows, idx = gmPG2.run_rw_queries (
-		queries = [
-			{'cmd': cmd, 'args': {'type': template_type, 'nshort': name_short, 'nlong': name_long, 'ext_version': 'new'}},
-			{'cmd': u"select currval(pg_get_serial_sequence('ref.paperwork_templates', 'pk'))"}
-		],
-		return_data = True
-	)
+	cmd = u"""
+		INSERT INTO ref.paperwork_templates (
+			fk_template_type,
+			name_short,
+			name_long,
+			external_version
+		) VALUES (
+			%(type)s,
+			%(nshort)s,
+			%(nlong)s,
+			%(ext_version)s
+		)
+		RETURNING pk
+	"""
+	args = {'type': template_type, 'nshort': name_short, 'nlong': name_long, 'ext_version': 'new'}
+	rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}], return_data = True)
 	template = cFormTemplate(aPK_obj = rows[0][0])
 	return template
+
 #------------------------------------------------------------
 def delete_form_template(template=None):
 	rows, idx = gmPG2.run_rw_queries (
-		queries = [
-			{'cmd': u'delete from ref.paperwork_templates where pk=%(pk)s', 'args': {'pk': template['pk_paperwork_template']}}
-		]
+		queries = [{
+			'cmd': u'DELETE FROM ref.paperwork_templates WHERE pk = %(pk)s',
+			'args': {'pk': template['pk_paperwork_template']}
+		}]
 	)
 	return True
+
 #============================================================
 # OpenOffice/LibreOffice API
 #============================================================
@@ -304,6 +325,9 @@ uno = None
 cOOoDocumentCloseListener = None
 writer_binary = None
 
+# http://forum.openoffice.org/en/forum/viewtopic.php?t=36370
+# http://stackoverflow.com/questions/4270962/using-pyuno-with-my-existing-python-installation
+
 #-----------------------------------------------------------
 def __configure_path_to_UNO():
 
@@ -457,7 +481,7 @@ class gmOOoConnector(gmBorg.cBorg):
 		dbcfg = gmCfg.cCfgSQL()
 		self.ooo_startup_settle_time = dbcfg.get2 (
 			option = u'external.ooo.startup_settle_time',
-			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 			bias = u'workplace',
 			default = 3.0
 		)
@@ -815,12 +839,37 @@ class cAbiWordForm(cFormEngine):
 		enc = sys.getfilesystemencoding()
 		cmd = (r'%s %s' % (self.abiword_binary, self.instance_filename.encode(enc))).encode(enc)
 		result = gmShellAPI.run_command_in_shell(command = cmd, blocking = True)
-		self.re_editable_filenames = []
+		self.re_editable_filenames = [self.instance_filename]
 		return result
 	#--------------------------------------------------------
 	def generate_output(self, instance_file=None, format=None):
-		self.final_output_filenames = [self.instance_filename]
-		return self.instance_filename
+
+		if instance_file is None:
+			instance_file = self.instance_filename
+		try:
+			open(instance_file, 'r').close()
+		except:
+			_log.exception('cannot access form instance file [%s]', instance_file)
+			gmLog2.log_stack_trace()
+			return None
+		self.instance_filename = instance_file
+
+		_log.debug('ignoring <format> directive [%s], generating PDF', format)
+
+		pdf_name = os.path.splitext(self.instance_filename)[0] + u'.pdf'
+		cmd = u'%s --to=pdf --to-name=%s %s' % (
+			self.abiword_binary,
+			pdf_name,
+			self.instance_filename
+		)
+		if not gmShellAPI.run_command_in_shell(command = cmd, blocking = True):
+			_log.error('problem running abiword, cannot generate form output')
+			gmDispatcher.send(signal = 'statustext', msg = _('Error running AbiWord. Cannot generate PDF.'), beep = True)
+			return None
+
+		self.final_output_filenames = [pdf_name]
+		return pdf_name
+
 #----------------------------------------------------------------
 form_engines[u'A'] = cAbiWordForm
 
@@ -907,13 +956,6 @@ class cTextForm(cFormEngine):
 				# also consider text *viewers* since pretty much any of them will be an editor as well
 				editor_cmd = gmMimeLib.get_viewer_cmd(mimetype, self.instance_filename)
 
-		# last resort
-#		if editor_cmd is None:
-#			if os.name == 'nt':
-#				editor_cmd = u'notepad.exe %s' % self.instance_filename
-#			else:
-#				editor_cmd = u'sensible-editor %s' % self.instance_filename
-
 		if editor_cmd is not None:
 			result = gmShellAPI.run_command_in_shell(command = editor_cmd, blocking = True)
 		self.re_editable_filenames = [self.instance_filename]
@@ -1011,26 +1053,33 @@ class cLaTeXForm(cFormEngine):
 
 		for line in template_file:
 
-			if line.strip() in [u'', u'\r', u'\n', u'\r\n']:
+			if line.strip() in [u'', u'\r', u'\n', u'\r\n']:		# empty lines
+				instance_file.write(line)
+				continue
+			if line.lstrip().startswith('%'):		# TeX comment
 				instance_file.write(line)
 				continue
 
-			# 1) find placeholders in this line
-			placeholders_in_line = regex.findall(data_source.placeholder_regex, line, regex.IGNORECASE)
-			if len(placeholders_in_line) > 0:
+			for placeholder_regex in [data_source.first_order_placeholder_regex, data_source.second_order_placeholder_regex, data_source.third_order_placeholder_regex]:
+				# 1) find placeholders in this line
+				placeholders_in_line = regex.findall(placeholder_regex, line, regex.IGNORECASE)
+				if len(placeholders_in_line) == 0:
+					continue
+				_log.debug('%s placeholders found with pattern: %s', len(placeholders_in_line), placeholder_regex)
 				found_placeholders = True
-			# 2) and replace them
-			for placeholder in placeholders_in_line:
-				try:
-					val = data_source[placeholder]
-				except:
-					val = _('error with placeholder [%s]') % gmTools.tex_escape_string(placeholder)
-					_log.exception(val)
+				# 2) replace them
+				for placeholder in placeholders_in_line:
+					try:
+						val = data_source[placeholder]
+					except:
+						_log.exception('error with placeholder [%s]', placeholder)
+						val = gmTools.tex_escape_string(_('error with placeholder [%s]') % placeholder)
 
-				if val is None:
-					val = _('error with placeholder [%s]') % gmTools.tex_escape_string(placeholder)
+					if val is None:
+						_log.debug('error with placeholder [%s]', placeholder)
+						val = _('error with placeholder [%s]') % gmTools.tex_escape_string(placeholder)
 
-				line = line.replace(placeholder, val)
+					line = line.replace(placeholder, val)
 
 			instance_file.write(line)
 
@@ -1061,14 +1110,6 @@ class cLaTeXForm(cFormEngine):
 				if editor_cmd is not None:
 					break
 
-		# last resort
-		# Vaibhav BANAIT protested against this
-#		if editor_cmd is None:
-#			if os.name == 'nt':
-#				editor_cmd = u'notepad.exe %s' % self.instance_filename
-#			else:
-#				editor_cmd = u'sensible-editor %s' % self.instance_filename
-
 		if editor_cmd is None:
 			return False
 
@@ -1094,11 +1135,11 @@ class cLaTeXForm(cFormEngine):
 
 		# LaTeX can need up to three runs to get cross references et al right
 		if platform.system() == 'Windows':
-			draft_cmd = r'pdflatex.exe -draftmode -interaction=nonstopmode -output-directory=%s %s' % (self.__sandbox_dir, self.instance_filename)
-			final_cmd = r'pdflatex.exe -interaction=nonstopmode -output-directory=%s %s' % (self.__sandbox_dir, self.instance_filename)
+			draft_cmd = r'pdflatex.exe -draftmode -recorder -interaction=nonstopmode -output-directory=%s %s' % (self.__sandbox_dir, self.instance_filename)
+			final_cmd = r'pdflatex.exe -recorder -interaction=nonstopmode -output-directory=%s %s' % (self.__sandbox_dir, self.instance_filename)
 		else:
-			draft_cmd = r'pdflatex -draftmode -interaction=nonstopmode -output-directory=%s %s' % (self.__sandbox_dir, self.instance_filename)
-			final_cmd = r'pdflatex -interaction=nonstopmode -output-directory=%s %s' % (self.__sandbox_dir, self.instance_filename)
+			draft_cmd = r'pdflatex -draftmode -recorder -interaction=nonstopmode -output-directory=%s %s' % (self.__sandbox_dir, self.instance_filename)
+			final_cmd = r'pdflatex -recorder -interaction=nonstopmode -output-directory=%s %s' % (self.__sandbox_dir, self.instance_filename)
 		for run_cmd in [draft_cmd, draft_cmd, final_cmd]:
 			if not gmShellAPI.run_command_in_shell(command = run_cmd, blocking = True, acceptable_return_codes = [0, 1]):
 				_log.error('problem running pdflatex, cannot generate form output')
@@ -1149,11 +1190,6 @@ class cXeTeXForm(cFormEngine):
 		super(self.__class__, self).__init__(template_file = template_file)
 
 		self.__sandbox_dir = sandbox_dir
-
-#		path, ext = os.path.splitext(self.template_filename)
-#		if ext in [r'', r'.']:
-#			ext = r'.tex'
-#		self.instance_filename = r'%s-instance%s' % (path, ext)
 	#--------------------------------------------------------
 	def substitute_placeholders(self, data_source=None):
 
@@ -1208,26 +1244,33 @@ class cXeTeXForm(cFormEngine):
 
 		for line in template_file:
 
-			if line.strip() in [u'', u'\r', u'\n', u'\r\n']:
+			if line.strip() in [u'', u'\r', u'\n', u'\r\n']:		# empty lines
+				instance_file.write(line)
+				continue
+			if line.startswith('%'):		# TeX comment
 				instance_file.write(line)
 				continue
 
-			# 1) find placeholders in this line
-			placeholders_in_line = regex.findall(data_source.placeholder_regex, line, regex.IGNORECASE)
-			if len(placeholders_in_line) > 0:
+			for placeholder_regex in [data_source.first_order_placeholder_regex, data_source.second_order_placeholder_regex, data_source.third_order_placeholder_regex]:
+				# 1) find placeholders in this line
+				placeholders_in_line = regex.findall(placeholder_regex, line, regex.IGNORECASE)
+				if len(placeholders_in_line) == 0:
+					continue
+				_log.debug('%s placeholders found with pattern: %s', len(placeholders_in_line), placeholder_regex)
 				found_placeholders = True
-			# 2) and replace them
-			for placeholder in placeholders_in_line:
-				try:
-					val = data_source[placeholder]
-				except:
-					val = _('error with placeholder [%s]') % gmTools.tex_escape_string(placeholder, replace_known_unicode=False)
-					_log.exception(val)
+				# 2) replace them
+				for placeholder in placeholders_in_line:
+					try:
+						val = data_source[placeholder]
+					except:
+						_log.exception('error with placeholder [%s]', placeholder)
+						val = gmTools.tex_escape_string(_('error with placeholder [%s]') % placeholder)
 
-				if val is None:
-					val = _('error with placeholder [%s]') % gmTools.tex_escape_string(placeholder, replace_known_unicode=False)
+					if val is None:
+						_log.debug('error with placeholder [%s]', placeholder)
+						val = _('error with placeholder [%s]') % gmTools.tex_escape_string(placeholder)
 
-				line = line.replace(placeholder, val)
+					line = line.replace(placeholder, val)
 
 			instance_file.write(line)
 
@@ -1259,13 +1302,6 @@ class cXeTeXForm(cFormEngine):
 				if editor_cmd is not None:
 					break
 
-		# last resort
-#		if editor_cmd is None:
-#			if os.name == 'nt':
-#				editor_cmd = u'notepad.exe %s' % self.instance_filename
-#			else:
-#				editor_cmd = u'sensible-editor %s' % self.instance_filename
-
 		if editor_cmd is None:
 			return False
 
@@ -1889,7 +1925,7 @@ def test_au():
 	'CLINICALNOTES':"""heard new murmur
 	Here's some
 crap to demonstrate how it can cover multiple lines.""",
-	'COPYADDRESS':'Karsten Hilbert\nLeipzig, Germany',
+	'COPYADDRESS':'Jack Jones\nHannover, Germany',
 	'ROUTINE':1,
 	'URGENT':0,
 	'FAX':1,
@@ -2128,9 +2164,9 @@ if __name__ == '__main__':
 
 	#test_cFormTemplate()
 	#set_template_from_file()
-	#test_latex_form()
+	test_latex_form()
 	#test_pdf_form()
 	#test_abiword_form()
-	test_text_form()
+	#test_text_form()
 
 #============================================================
