@@ -17,11 +17,11 @@ from Gnumed.pycommon import gmCfg2
 from Gnumed.pycommon import gmBusinessDBObject
 
 from Gnumed.business import gmOrganization
-from Gnumed.business import gmDemographicRecord
 
 
 _log = logging.getLogger('gm.praxis')
 _cfg = gmCfg2.gmCfgData()
+
 #============================================================
 def delete_workplace(workplace=None, delete_config=False, conn=None):
 
@@ -94,6 +94,9 @@ class cPraxisBranch(gmBusinessDBObject.cBusinessDBObject):
 	def get_comm_channels(self, comm_medium=None):
 		return self.org_unit.get_comm_channels(comm_medium = comm_medium)
 	#--------------------------------------------------------
+	def get_external_ids(self, id_type=None, issuer=None):
+		return self.org_unit.get_external_ids(id_type = id_type, issuer = issuer)
+	#--------------------------------------------------------
 	# properties
 	#--------------------------------------------------------
 	def _get_org_unit(self):
