@@ -9,6 +9,7 @@ __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 
 
 import sys
+import logging
 
 
 if __name__ == '__main__':
@@ -20,6 +21,8 @@ from Gnumed.pycommon import gmDateTime
 
 from Gnumed.business import gmStaff
 
+_log = logging.getLogger('gm.inbox')
+
 #============================================================
 # provider message inbox
 #------------------------------------------------------------
@@ -136,6 +139,8 @@ def get_reminders(pk_patient=None, order_by=None):
 	cmd = u"SELECT * FROM dem.v_message_inbox WHERE %s" % (
 		order_by % u' AND '.join(where_parts)
 	)
+	_log.debug('SQL: %s', cmd)
+	_log.debug('args: %s', args)
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 
 	return [ cInboxMessage(row = {'data': r, 'idx': idx, 'pk_field': 'pk_inbox_message'}) for r in rows ]
@@ -157,6 +162,47 @@ def get_overdue_messages(pk_patient=None, order_by=None):
 	cmd = u"SELECT * FROM dem.v_message_inbox WHERE %s" % (
 		order_by % u' AND '.join(where_parts)
 	)
+	_log.debug('SQL: %s', cmd)
+	_log.debug('args: %s', args)
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+
+	return [ cInboxMessage(row = {'data': r, 'idx': idx, 'pk_field': 'pk_inbox_message'}) for r in rows ]
+
+#------------------------------------------------------------
+def get_relevant_messages(pk_staff=None, pk_patient=None, include_without_provider=False, order_by=None):
+
+	if order_by is None:
+		order_by = u'%s ORDER BY importance desc, received_when desc'
+	else:
+		order_by = u'%%s ORDER BY %s' % order_by
+
+	args = {}
+	where_parts = []
+
+	if pk_staff is not None:
+		if include_without_provider:
+			where_parts.append(u'((pk_staff IN (%(staff)s, NULL)) OR (modified_by = (SELECT short_alias FROM dem.staff WHERE pk = %(staff)s)))')
+		else:
+			where_parts.append(u'((pk_staff = %(staff)s) OR (modified_by = (SELECT short_alias FROM dem.staff WHERE pk = %(staff)s)))')
+		args['staff'] = pk_staff
+
+	if pk_patient is not None:
+		where_parts.append(u'pk_patient = %(pat)s')
+		args['pat'] = pk_patient
+
+	where_parts.append(u"""
+		-- messages which have no due date and are not expired
+		((due_date IS NULL) AND ((expiry_date IS NULL) OR (expiry_date > now())))
+			OR
+		-- messages which are due and not expired
+		((due_date IS NOT NULL) AND (due_date < now()) AND ((expiry_date IS NULL) OR (expiry_date > now())))
+	""")
+
+	cmd = _SQL_get_inbox_messages % (
+		order_by % u' AND '.join(where_parts)
+	)
+	_log.debug('SQL: %s', cmd)
+	_log.debug('args: %s', args)
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 
 	return [ cInboxMessage(row = {'data': r, 'idx': idx, 'pk_field': 'pk_inbox_message'}) for r in rows ]
@@ -201,6 +247,8 @@ def get_inbox_messages(pk_staff=None, pk_patient=None, include_without_provider=
 	cmd = _SQL_get_inbox_messages % (
 		order_by % u' AND '.join(where_parts)
 	)
+	_log.debug('SQL: %s', cmd)
+	_log.debug('args: %s', args)
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 
 	return [ cInboxMessage(row = {'data': r, 'idx': idx, 'pk_field': 'pk_inbox_message'}) for r in rows ]
@@ -315,40 +363,54 @@ class cProviderInbox:
 			order_by = order_by
 		)
 
-	def _set_messages(self, messages):
-		return
+	messages = property(get_messages, lambda x:x)
 
-	messages = property(get_messages, _set_messages)
+	#--------------------------------------------------------
+	def get_relevant_messages(self, pk_patient=None, include_without_provider=True):
+		return get_relevant_messages (
+			pk_staff = self.__provider_id,
+			pk_patient = pk_patient,
+			include_without_provider = include_without_provider
+		)
 
 #============================================================
 # dynamic hints API
 #------------------------------------------------------------
-_SQL_get_dynamic_hints = u"SELECT *, xmin AS xmin_auto_hint FROM ref.auto_hint WHERE %s"
+_SQL_get_dynamic_hints = u"SELECT * FROM ref.v_auto_hints WHERE %s"
 
 class cDynamicHint(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents dynamic hints to be run against the database."""
 
-	_cmd_fetch_payload = _SQL_get_dynamic_hints % u"pk = %s"
+	_cmd_fetch_payload = _SQL_get_dynamic_hints % u"pk_auto_hint = %s"
 	_cmds_store_payload = [
 		u"""UPDATE ref.auto_hint SET
+				query = gm.nullify_empty_string(%(query)s),
+				title = gm.nullify_empty_string(%(title)s),
+				hint = gm.nullify_empty_string(%(hint)s),
+				url = gm.nullify_empty_string(%(url)s),
+				source = gm.nullify_empty_string(%(source)s),
 				is_active = %(is_active)s
 			WHERE
-				pk = %(pk)s
+				pk = %(pk_auto_hint)s
 					AND
 				xmin = %(xmin_auto_hint)s
 			RETURNING
-				pk,
 				xmin AS xmin_auto_hint
 		"""
 	]
 	_updatable_fields = [
+		u'query',
+		u'title',
+		u'hint',
+		u'url',
+		u'source',
 		u'is_active'
 	]
 	#--------------------------------------------------------
 	def format(self):
 		txt = u'%s               [#%s]\n' % (
 			gmTools.bool2subst(self._payload[self._idx['is_active']], _('Active clinical hint'), _('Inactive clinical hint')),
-			self._payload[self._idx['pk']]
+			self._payload[self._idx['pk_auto_hint']]
 		)
 		txt += u'\n'
 		txt += u'%s\n\n' % self._payload[self._idx['title']]
@@ -366,49 +428,68 @@ class cDynamicHint(gmBusinessDBObject.cBusinessDBObject):
 		txt += u'\n'
 		txt += u'%s\n' % gmTools.wrap(self._payload[self._idx['query']], width = 50, initial_indent = u' ', subsequent_indent = u' ')
 		return txt
+	#--------------------------------------------------------
+	def suppress(self, rationale=None, pk_encounter=None):
+		return suppress_dynamic_hint (
+			pk_hint = self._payload[self._idx['pk_auto_hint']],
+			pk_encounter = pk_encounter,
+			rationale = rationale
+		)
+	#--------------------------------------------------------
+	def invalidate_suppression(self, pk_encounter=None):
+		return invalidate_hint_suppression (
+			pk_hint = self._payload[self._idx['pk_auto_hint']],
+			pk_encounter = pk_encounter
+		)
 
 #------------------------------------------------------------
-def get_dynamic_hints(order_by=None):
+def get_dynamic_hints(order_by=None, link_obj=None):
 	if order_by is None:
 		order_by = u'true'
 	else:
 		order_by = u'true ORDER BY %s' % order_by
 
 	cmd = _SQL_get_dynamic_hints % order_by
-	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}], get_col_idx = True)
-	return [ cDynamicHint(row = {'data': r, 'idx': idx, 'pk_field': 'pk'}) for r in rows ]
-#------------------------------------------------------------
-#def create_xxx(xxx=None, xxx=None):
-#
-#	args = {
-#		u'xxx': xxx,
-#		u'xxx': xxx
-#	}
-#	cmd = u"""
-#		INSERT INTO xxx.xxx (
-#			xxx,
-#			xxx,
-#			xxx
-#		) VALUES (
-#			%(xxx)s,
-#			%(xxx)s,
-#			gm.nullify_empty_string(%(xxx)s)
-#		)
-#		RETURNING pk
-#	"""
-#	rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}], return_data = True, get_col_idx = False)
-#
-#	return cDynamicHint(aPK_obj = rows[0]['pk'])
-#------------------------------------------------------------
-#def delete_xxx(xxx=None):
-#	args = {'pk': xxx}
-#	cmd = u"DELETE FROM xxx.xxx WHERE pk = %(pk)s"
-#	gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
-#	return True
-#------------------------------------------------------------
+	rows, idx = gmPG2.run_ro_queries(link_obj = link_obj, queries = [{'cmd': cmd}], get_col_idx = True)
+	return [ cDynamicHint(row = {'data': r, 'idx': idx, 'pk_field': 'pk_auto_hint'}) for r in rows ]
 
 #------------------------------------------------------------
-def get_hints_for_patient(pk_identity=None):
+def create_dynamic_hint(link_obj=None, query=None, title=None, hint=None, source=None):
+	args = {
+		u'query': query,
+		u'title': title,
+		u'hint': hint,
+		u'source': source,
+		u'usr': gmStaff.gmCurrentProvider()['db_user']
+	}
+	cmd = u"""
+		INSERT INTO ref.auto_hint (
+			query,
+			title,
+			hint,
+			source,
+			lang
+		) VALUES (
+			gm.nullify_empty_string(%(query)s),
+			gm.nullify_empty_string(%(title)s),
+			gm.nullify_empty_string(%(hint)s),
+			gm.nullify_empty_string(%(source)s),
+			i18n.get_curr_lang(%(usr)s)
+		)
+		RETURNING pk
+	"""
+	rows, idx = gmPG2.run_rw_queries(link_obj = link_obj, queries = [{'cmd': cmd, 'args': args}], return_data = True, get_col_idx = True)
+	return cDynamicHint(aPK_obj = rows[0]['pk'], link_obj = link_obj)
+
+#------------------------------------------------------------
+def delete_dynamic_hint(link_obj=None, pk_hint=None):
+	args = {'pk': pk_hint}
+	cmd = u"DELETE FROM ref.auto_hint WHERE pk = %(pk)s"
+	gmPG2.run_rw_queries(link_obj = link_obj, queries = [{'cmd': cmd, 'args': args}])
+	return True
+
+#------------------------------------------------------------
+def get_hints_for_patient(pk_identity=None, include_suppressed_needing_invalidation=False):
 	conn = gmPG2.get_connection()
 	curs = conn.cursor()
 	curs.callproc('clin.get_hints_for_patient', [pk_identity])
@@ -416,7 +497,132 @@ def get_hints_for_patient(pk_identity=None):
 	idx = gmPG2.get_col_indices(curs)
 	curs.close()
 	conn.rollback()
-	return [ cDynamicHint(row = {'data': r, 'idx': idx, 'pk_field': 'pk'}) for r in rows ]
+	if not include_suppressed_needing_invalidation:
+		return [ cDynamicHint(row = {'data': r, 'idx': idx, 'pk_field': 'pk_auto_hint'}) for r in rows if r['rationale4suppression'] != 'please_invalidate_suppression' ]
+	return [ cDynamicHint(row = {'data': r, 'idx': idx, 'pk_field': 'pk_auto_hint'}) for r in rows ]
+
+#------------------------------------------------------------
+def suppress_dynamic_hint(pk_hint=None, rationale=None, pk_encounter=None):
+	args = {
+		'hint': pk_hint,
+		'rationale': rationale,
+		'enc': pk_encounter
+	}
+	cmd = u"""
+		DELETE FROM clin.suppressed_hint
+		WHERE
+			fk_hint = %(hint)s
+				AND
+			fk_encounter IN (
+				SELECT pk FROM clin.encounter WHERE fk_patient = (
+					SELECT fk_patient FROM clin.encounter WHERE pk = %(enc)s
+				)
+			)
+	"""
+	queries = [{'cmd': cmd, 'args': args}]
+	cmd = u"""
+		INSERT INTO clin.suppressed_hint (
+			fk_encounter,
+			fk_hint,
+			rationale,
+			md5_sum
+		) VALUES (
+			%(enc)s,
+			%(hint)s,
+			%(rationale)s,
+			(SELECT r_vah.md5_sum FROM ref.v_auto_hints r_vah WHERE r_vah.pk_auto_hint = %(hint)s)
+		)
+	"""
+	queries.append({'cmd': cmd, 'args': args})
+	gmPG2.run_rw_queries(queries = queries)
+	return True
+
+#------------------------------------------------------------
+# suppressed dynamic hints
+#------------------------------------------------------------
+_SQL_get_suppressed_hints = u"SELECT * FROM clin.v_suppressed_hints WHERE %s"
+
+class cSuppressedHint(gmBusinessDBObject.cBusinessDBObject):
+	"""Represents suppressed dynamic hints per patient."""
+
+	_cmd_fetch_payload = _SQL_get_suppressed_hints % u"pk_suppressed_hint = %s"
+	_cmds_store_payload = []
+	_updatable_fields = []
+	#--------------------------------------------------------
+	def format(self):
+		txt = u'%s               [#%s]\n' % (
+			gmTools.bool2subst(self._payload[self._idx['is_active']], _('Suppressed active dynamic hint'), _('Suppressed inactive dynamic hint')),
+			self._payload[self._idx['pk_suppressed_hint']]
+		)
+		txt += u'\n'
+		txt += u'%s\n\n' % self._payload[self._idx['title']]
+		txt += _('Suppressed by: %s\n') % self._payload[self._idx['suppressed_by']]
+		txt += _('Suppressed at: %s\n') % gmDateTime.pydt_strftime(self._payload[self._idx['suppressed_when']], '%Y %b %d')
+		txt += _('Hint #: %s\n') % self._payload[self._idx['pk_hint']]
+		txt += _('Patient #: %s\n') % self._payload[self._idx['pk_identity']]
+		txt += _('MD5 (currently): %s\n') % self._payload[self._idx['md5_hint']]
+		txt += _('MD5 (at suppression): %s\n') % self._payload[self._idx['md5_suppressed']]
+		txt += _('Source: %s\n') % self._payload[self._idx['source']]
+		txt += _('Language: %s\n') % self._payload[self._idx['lang']]
+		txt += u'\n'
+		txt += u'%s\n' % gmTools.wrap(self._payload[self._idx['hint']], width = 50, initial_indent = u' ', subsequent_indent = u' ')
+		txt += u'\n'
+		txt += u'%s\n' % gmTools.wrap (
+			gmTools.coalesce(self._payload[self._idx['url']], u''),
+			width = 50,
+			initial_indent = u' ',
+			subsequent_indent = u' '
+		)
+		txt += u'\n'
+		txt += u'%s\n' % gmTools.wrap(self._payload[self._idx['query']], width = 50, initial_indent = u' ', subsequent_indent = u' ')
+		return txt
+
+#------------------------------------------------------------
+def get_suppressed_hints(pk_identity=None, order_by=None):
+	args = {'pat': pk_identity}
+	if pk_identity is None:
+		where = u'true'
+	else:
+		where = u"pk_identity = %(pat)s"
+	if order_by is not None:
+		order_by = u' ORDER BY %s' % order_by
+	cmd = (_SQL_get_suppressed_hints % where) + order_by
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+	return [ cSuppressedHint(row = {'data': r, 'idx': idx, 'pk_field': 'pk_suppressed_hint'}) for r in rows ]
+
+#------------------------------------------------------------
+def delete_suppressed_hint(pk_suppressed_hint=None):
+	args = {'pk': pk_suppressed_hint}
+	cmd = u"DELETE FROM clin.suppressed_hint WHERE pk = %(pk)s"
+	gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+	return True
+
+#------------------------------------------------------------
+def invalidate_hint_suppression(pk_hint=None, pk_encounter=None):
+	_log.debug('invalidating suppression of hint #%s', pk_hint)
+	args = {
+		'pk_hint': pk_hint,
+		'enc': pk_encounter,
+		'fake_md5': '***INVALIDATED***'			# only needs to NOT match ANY md5 sum
+	}
+	cmd = u"""
+		UPDATE clin.suppressed_hint SET
+			fk_encounter = %(enc)s,
+			md5_sum = %(fake_md5)s
+		WHERE
+			pk = (
+				SELECT pk_suppressed_hint
+				FROM clin.v_suppressed_hints
+				WHERE
+					pk_hint = %(pk_hint)s
+						AND
+					pk_identity = (
+						SELECT fk_patient FROM clin.encounter WHERE pk = %(enc)s
+					)
+			)
+		"""
+	gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+	return True
 
 #============================================================
 if __name__ == '__main__':
@@ -451,7 +657,9 @@ if __name__ == '__main__':
 			print msg.format()
 	#---------------------------------------
 	def test_auto_hints():
-		for row in get_dynamic_hints(pk_identity = 13):
+#		for row in get_dynamic_hints():
+#			print row
+		for row in get_hints_for_patient(pk_identity = 12):
 			print row
 	#---------------------------------------
 	#test_inbox()
