@@ -29,7 +29,8 @@ fi
 
 
 # standard options
-LOG="--log-file=gm-from-vcs-$$.log"
+TS=`date +%m_%d-%H%M%S`
+LOG="--log-file=gm-vcs-${TS}-$$.log"
 CONF="--conf-file=gm-from-vcs.conf"
 
 # options useful for development and debugging:
@@ -48,10 +49,10 @@ TARBALL_OPTS="--local-import --debug"
 echo "-------------------------------------------------"
 echo "Running from Git branch: "`git branch | grep \*`
 echo "-------------------------------------------------"
-python gnumed.py ${LOG} ${CONF} ${DEV_OPTS} $@
+#python gnumed.py ${LOG} ${CONF} ${DEV_OPTS} $@
 
 # - released tarball version:
-#python gnumed.py ${LOG} ${CONF} ${TARBALL_OPTS} $@
+python gnumed.py ${LOG} ${CONF} ${TARBALL_OPTS} $@
 
 # - production version:
 #python gnumed.py ${LOG} ${CONF} $@
