@@ -249,9 +249,11 @@ def format_dob(dob, format='%Y %b %d', encoding=None, none_string=None, dob_is_e
 
 	return dob_txt
 #---------------------------------------------------------------------------
-def pydt_strftime(dt=None, format='%Y %b %d  %H:%M.%S', encoding=None, accuracy=None):
+def pydt_strftime(dt=None, format='%Y %b %d  %H:%M.%S', encoding=None, accuracy=None, none_str=None):
 
 	if dt is None:
+		if none_str is not None:
+			return none_str
 		dt = pydt_now_here()
 
 	if encoding is None:
@@ -262,6 +264,9 @@ def pydt_strftime(dt=None, format='%Y %b %d  %H:%M.%S', encoding=None, accuracy=
 	except ValueError:
 		_log.exception('Python cannot strftime() this <datetime>')
 
+	if isinstance(dt, pyDT.date):
+		accuracy = acc_days
+
 	if accuracy == acc_days:
 		return u'%04d-%02d-%02d' % (
 			dt.year,
@@ -554,6 +559,25 @@ def format_interval_medically(interval=None):
 	if seconds == 0:
 		return '0:%02d' % int(minutes)
 	return "%s.%ss" % (int(minutes), int(seconds))
+
+#---------------------------------------------------------------------------
+def format_pregnancy_weeks(age):
+	weeks, days = divmod(age.days, 7)
+	return u'%s%s%s%s' % (
+		int(weeks),
+		_('interval_format_tag::weeks::w')[-1:],
+		interval.days,
+		_('interval_format_tag::days::d')[-1:]
+	)
+
+#---------------------------------------------------------------------------
+def format_pregnancy_months(age):
+	months, remainder = divmod(age.days, 28)
+	return u'%s%s' % (
+		int(months) + 1,
+		_('interval_format_tag::months::m')[-1:]
+	)
+
 #---------------------------------------------------------------------------
 def is_leap_year(year):
 	# year is multiple of 4 ?
@@ -2364,6 +2388,7 @@ if __name__ == '__main__':
 	#-------------------------------------------------
 	def test_pydt_strftime():
 		dt = pydt_now_here()
+		print pydt_strftime(dt, '-(%Y %b %d)-')
 		print pydt_strftime(dt)
 		print pydt_strftime(dt, accuracy = acc_days)
 		print pydt_strftime(dt, accuracy = acc_minutes)
@@ -2391,8 +2416,8 @@ if __name__ == '__main__':
 	#test_str2interval()
 	#test_format_interval()
 	#test_format_interval_medically()
-	test_str2pydt()
-	#test_pydt_strftime()
+	#test_str2pydt()
+	test_pydt_strftime()
 	#test_calculate_apparent_age()
 	#test_is_leap_year()
 
