@@ -535,7 +535,7 @@ def create_invoice_from_bill(parent = None, bill=None, print_it=False, keep_a_co
 			)
 			return False
 
-	if None in [ bill['close_date'], bill['pk_receiver_address'] ]:
+	if None in [ bill['close_date'], bill['pk_receiver_address'], bill['apply_vat'] ]:
 		edit_bill(parent = parent, bill = bill, single_entry = True)
 		# cannot invoice open bills
 		if bill['close_date'] is None:
@@ -561,6 +561,18 @@ def create_invoice_from_bill(parent = None, bill=None, print_it=False, keep_a_co
 				)
 			)
 			return False
+		# cannot create invoice if applying VAT is undecided
+		if bill['apply_vat'] is None:
+			_log.error('cannot create invoice from bill, apply_vat undecided')
+			gmGuiHelpers.gm_show_warning (
+				aTitle = _('Creating invoice'),
+				aMessage = _(
+					'Cannot create invoice from bill.\n'
+					'\n'
+					'You must decide on whether to apply VAT.'
+				)
+			)
+			return False
 
 	# find template
 	template = get_invoice_template(parent = parent, with_vat = bill['apply_vat'])
@@ -869,7 +881,8 @@ def manage_bills(parent=None, patient=None):
 				amount = gmTools.bool2subst (
 					b['apply_vat'],
 					_('%(currency)s%(total_amount_with_vat)s (with %(percent_vat)s%% VAT)') % b,
-					u'%(currency)s%(total_amount)s' % b
+					u'%(currency)s%(total_amount)s' % b,
+					_('without VAT: %(currency)s%(total_amount)s / with %(percent_vat)s%% VAT: %(currency)s%(total_amount_with_vat)s') % b
 				)
 			items.append ([
 				close_date,
@@ -922,6 +935,17 @@ class cBillEAPnl(wxgBillEAPnl.wxgBillEAPnl, gmEditArea.cGenericEditAreaMixin):
 		if data is not None:
 			self.mode = 'edit'
 
+		self._3state2bool = {
+			wx.CHK_UNCHECKED: False,
+			wx.CHK_CHECKED: True,
+			wx.CHK_UNDETERMINED: None
+		}
+		self.bool_to_3state = {
+			False: wx.CHK_UNCHECKED,
+			True: wx.CHK_CHECKED,
+			None: wx.CHK_UNDETERMINED
+		}
+
 #		self.__init_ui()
 	#----------------------------------------------------------------
 #	def __init_ui(self):
@@ -935,6 +959,11 @@ class cBillEAPnl(wxgBillEAPnl.wxgBillEAPnl, gmEditArea.cGenericEditAreaMixin):
 		if not self._PRW_close_date.is_valid_timestamp(allow_empty = False):
 			self._PRW_close_date.SetFocus()
 
+		# flag but do not count as wrong
+		if self._CHBOX_vat_applies.ThreeStateValue == wx.CHK_UNDETERMINED:
+			self._CHBOX_vat_applies.SetFocus()
+			self._CHBOX_vat_applies.SetBackgroundColour('yellow')
+
 		return validity
 	#----------------------------------------------------------------
 	def _save_as_new(self):
@@ -943,7 +972,7 @@ class cBillEAPnl(wxgBillEAPnl.wxgBillEAPnl, gmEditArea.cGenericEditAreaMixin):
 	#----------------------------------------------------------------
 	def _save_as_update(self):
 		self.data['close_date'] = self._PRW_close_date.GetData()
-		self.data['apply_vat'] = self._CHBOX_vat_applies.GetValue()
+		self.data['apply_vat'] = self._3state2bool[self._CHBOX_vat_applies.ThreeStateValue]
 		self.data['comment'] = self._TCTRL_comment.GetValue()
 		self.data.save()
 		return True
@@ -966,15 +995,17 @@ class cBillEAPnl(wxgBillEAPnl.wxgBillEAPnl, gmEditArea.cGenericEditAreaMixin):
 			self._TCTRL_address.SetValue(adr.format(single_line = True, show_type = False))
 
 		self._TCTRL_value.SetValue(u'%(currency)s%(total_amount)s' % self.data)
-		self._CHBOX_vat_applies.SetValue(self.data['apply_vat'])
+		self._CHBOX_vat_applies.ThreeStateValue = self.bool_to_3state[self.data['apply_vat']]
 		self._CHBOX_vat_applies.SetLabel(_('&VAT applies (%s%%)') % self.data['percent_vat'])
-		if self.data['apply_vat']:
+		if self.data['apply_vat'] is True:
 			tmp = u'%s %%(currency)s%%(total_vat)s %s %s %%(currency)s%%(total_amount_with_vat)s' % (
 				gmTools.u_corresponds_to,
 				gmTools.u_right_arrow,
 				gmTools.u_sum,
 			)
 			self._TCTRL_value_with_vat.SetValue(tmp % self.data)
+		elif self.data['apply_vat'] is None:
+			self._TCTRL_value_with_vat.SetValue(u'?')
 		else:
 			self._TCTRL_value_with_vat.SetValue(u'')
 
@@ -985,7 +1016,7 @@ class cBillEAPnl(wxgBillEAPnl.wxgBillEAPnl, gmEditArea.cGenericEditAreaMixin):
 	# event handling
 	#----------------------------------------------------------------
 	def _on_vat_applies_box_checked(self, event):
-		if self._CHBOX_vat_applies.GetValue():
+		if self._CHBOX_vat_applies.ThreeStateValue == wx.CHK_CHECKED:
 			tmp = u'%s %%(currency)s%%(total_vat)s %s %s %%(currency)s%%(total_amount_with_vat)s' % (
 				gmTools.u_corresponds_to,
 				gmTools.u_right_arrow,
@@ -993,6 +1024,9 @@ class cBillEAPnl(wxgBillEAPnl.wxgBillEAPnl, gmEditArea.cGenericEditAreaMixin):
 			)
 			self._TCTRL_value_with_vat.SetValue(tmp % self.data)
 			return
+		if self._CHBOX_vat_applies.ThreeStateValue == wx.CHK_UNDETERMINED:
+			self._TCTRL_value_with_vat.SetValue(u'?')
+			return
 		self._TCTRL_value_with_vat.SetValue(u'')
 	#----------------------------------------------------------------
 	def _on_select_address_button_pressed(self, event):
@@ -1455,21 +1489,21 @@ class cBillingPluginPnl(wxgBillingPluginPnl.wxgBillingPluginPnl, gmRegetMixin.cR
 	# event handling
 	#-----------------------------------------------------
 	def __register_interests(self):
-		gmDispatcher.connect(signal = u'pre_patient_selection', receiver = self._on_pre_patient_selection)
+		gmDispatcher.connect(signal = u'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
 		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
 
 		gmDispatcher.connect(signal = u'bill.bill_item_mod_db', receiver = self._on_bill_item_modified)
 
 		self._PRW_billable.add_callback_on_selection(self._on_billable_selected_in_prw)
 	#-----------------------------------------------------
-	def _on_pre_patient_selection(self):
-		wx.CallAfter(self.__reset_ui)
+	def _on_pre_patient_unselection(self):
+		self.__reset_ui()
 	#-----------------------------------------------------
 	def _on_post_patient_selection(self):
-		wx.CallAfter(self._schedule_data_reget)
+		self._schedule_data_reget()
 	#-----------------------------------------------------
 	def _on_bill_item_modified(self):
-		wx.CallAfter(self._schedule_data_reget)
+		self._schedule_data_reget()
 	#-----------------------------------------------------
 	def _on_non_invoiced_only_checkbox_toggled(self, event):
 		self._PNL_bill_items.show_non_invoiced_only = self._CHBOX_show_non_invoiced_only.GetValue()
