@@ -295,7 +295,7 @@ class cEditDocumentTypesPnl(wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl):
 		gmDispatcher.connect(signal = u'blobs.doc_type_mod_db', receiver = self._on_doc_type_mod_db)
 	#--------------------------------------------------------
 	def _on_doc_type_mod_db(self):
-		wx.CallAfter(self.repopulate_ui)
+		self.repopulate_ui()
 	#--------------------------------------------------------
 	def repopulate_ui(self):
 
@@ -356,7 +356,7 @@ class cEditDocumentTypesPnl(wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl):
 	def _on_set_translation_button_pressed(self, event):
 		doc_type = self._LCTRL_doc_type.get_selected_item_data()
 		if doc_type.set_translation(translation = self._TCTRL_l10n_type.GetValue().strip()):
-			wx.CallAfter(self.repopulate_ui)
+			self.repopulate_ui()
 
 		return
 	#--------------------------------------------------------
@@ -832,7 +832,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 	#--------------------------------------------------------
 	# patient change plugin API
 	#--------------------------------------------------------
-	def _pre_patient_selection(self, **kwds):
+	def _pre_patient_unselection(self, **kwds):
 		# FIXME: persist pending data from here
 		pass
 	#--------------------------------------------------------
@@ -1031,7 +1031,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 			defaultDir = os.path.expanduser(os.path.join('~', 'gnumed')),
 			defaultFile = '',
 			wildcard = "%s (*)|*|TIFFs (*.tif)|*.tif|JPEGs (*.jpg)|*.jpg|%s (*.*)|*.*" % (_('all files'), _('all files (Win)')),
-			style = wx.OPEN | wx.HIDE_READONLY | wx.FILE_MUST_EXIST | wx.MULTIPLE
+			style = wx.OPEN | wx.FILE_MUST_EXIST | wx.MULTIPLE
 		)
 		result = dlg.ShowModal()
 		if result != wx.ID_CANCEL:
@@ -1041,6 +1041,16 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 			self.__reload_LBOX_doc_pages()
 		dlg.Destroy()
 	#--------------------------------------------------------
+	def _clipboard_btn_pressed(self, event):
+		event.Skip()
+		clip = gmGuiHelpers.clipboard2file()
+		if clip is None:
+			return
+		if clip is False:
+			return
+		self.acquired_pages.append(clip)
+		self.__reload_LBOX_doc_pages()
+	#--------------------------------------------------------
 	def _show_btn_pressed(self, evt):
 		# did user select a page ?
 		page_idx = self._LBOX_doc_pages.GetSelection()
@@ -1500,7 +1510,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin, treemixin.Expansion
 
 #		 wx.EVT_LEFT_DCLICK(self.tree, self.OnLeftDClick)
 
-		gmDispatcher.connect(signal = u'pre_patient_selection', receiver = self._on_pre_patient_selection)
+		gmDispatcher.connect(signal = u'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
 		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
 		gmDispatcher.connect(signal = u'blobs.doc_med_mod_db', receiver = self._on_doc_mod_db)
 		gmDispatcher.connect(signal = u'blobs.doc_obj_mod_db', receiver = self._on_doc_page_mod_db)
@@ -1560,6 +1570,9 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin, treemixin.Expansion
 		item = self.__doc_context_menu.Append(-1, _('Add parts'))
 		self.Bind(wx.EVT_MENU, self.__add_part, item)
 
+		item = self.__doc_context_menu.Append(-1, _('Add part from clipboard'))
+		self.Bind(wx.EVT_MENU, self.__add_part_from_clipboard, item)
+
 		ID = wx.NewId()
 		self.__doc_context_menu.Append(ID, _('Print all parts'))
 		wx.EVT_MENU(self.__doc_context_menu, ID, self.__print_doc)
@@ -1904,13 +1917,13 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin, treemixin.Expansion
 	#------------------------------------------------------------------------
 	def _on_doc_mod_db(self, *args, **kwargs):
 		self.__expanded_nodes = self.ExpansionState
-		wx.CallAfter(self._schedule_data_reget)
+		self._schedule_data_reget()
 	#------------------------------------------------------------------------
 	def _on_doc_page_mod_db(self, *args, **kwargs):
 		self.__expanded_nodes = self.ExpansionState
-		wx.CallAfter(self._schedule_data_reget)
+		self._schedule_data_reget()
 	#------------------------------------------------------------------------
-	def _on_pre_patient_selection(self, *args, **kwargs):
+	def _on_pre_patient_unselection(self, *args, **kwargs):
 		# empty out tree
 		if self.root is not None:
 			self.DeleteAllItems()
@@ -2203,7 +2216,10 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin, treemixin.Expansion
 
 		part_file = self.__curr_node_data.export_to_file(aChunkSize = chunksize)
 
-		cmd = u'%s %s' % (external_cmd, part_file)
+		if action == 'print':
+			cmd = u'%s generic_document %s' % (external_cmd, part_file)
+		else:
+			cmd = u'%s %s' % (external_cmd, part_file)
 		if os.name == 'nt':
 			blocking = True
 		else:
@@ -2350,7 +2366,15 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin, treemixin.Expansion
 			blocking = True
 		else:
 			blocking = False
-		cmd = external_cmd + u' ' + u' '.join(part_files)
+
+		if action == 'print':
+			cmd = u'%s %s %s' % (
+				external_cmd,
+				u'generic_document',
+				u' '.join(part_files)
+			)
+		else:
+			cmd = external_cmd + u' ' + u' '.join(part_files)
 		success = gmShellAPI.run_command_in_shell (
 			command = cmd,
 			blocking = blocking
@@ -2396,6 +2420,21 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin, treemixin.Expansion
 			self.__curr_node_data.add_parts_from_files(files = dlg.GetPaths(), reviewer = gmStaff.gmCurrentProvider()['pk_staff'])
 		dlg.Destroy()
 	#--------------------------------------------------------
+	def __add_part_from_clipboard(self, evt):
+		clip = gmGuiHelpers.clipboard2file()
+		if clip is None:
+			return
+		if clip is False:
+			return
+		gmMimeLib.call_viewer_on_file(clip, block = False)
+		really_add = gmGuiHelpers.gm_show_question (
+			question = _('Really add the displayed clipboard item into the document ?'),
+			title = _('Document part from clipboard')
+		)
+		if not really_add:
+			return
+		self.__curr_node_data.add_parts_from_files(files = [clip], reviewer = gmStaff.gmCurrentProvider()['pk_staff'])
+	#--------------------------------------------------------
 	def __access_external_original(self, evt):
 
 		gmHooks.run_hook_script(hook = u'before_external_doc_access')
