@@ -26,6 +26,8 @@ from Gnumed.pycommon import gmMimeLib
 
 from Gnumed.business import gmForms
 from Gnumed.business import gmPerson
+from Gnumed.business import gmExternalCare
+from Gnumed.business import gmPraxis
 
 from Gnumed.wxpython import gmGuiHelpers
 from Gnumed.wxpython import gmListWidgets
@@ -36,12 +38,10 @@ from Gnumed.wxpython.gmDocumentWidgets import save_files_as_new_document
 
 _log = logging.getLogger('gm.ui')
 
-_ID_FORM_DISPOSAL_PRINT_NOW, \
-_ID_FORM_DISPOSAL_MAIL_NOW, \
-_ID_FORM_DISPOSAL_FAX_NOW, \
-_ID_FORM_DISPOSAL_TRAY_NOW, \
-_ID_FORM_DISPOSAL_ARCHIVE_NOW, \
-_ID_FORM_DISPOSAL_SAVE_NOW = range(6)
+_ID_FORM_DISPOSAL_PRINT, \
+_ID_FORM_DISPOSAL_REMOTE_PRINT, \
+_ID_FORM_DISPOSAL_EXPORT_ONLY, \
+_ID_FORM_DISPOSAL_ARCHIVE_ONLY = range(4)
 
 #============================================================
 # generic form generation and handling convenience functions
@@ -187,7 +187,10 @@ def generate_form_from_template(parent=None, template_types=None, edit=None, tem
 
 #------------------------------------------------------------
 def act_on_generated_forms(parent=None, forms=None, jobtype=None, episode_name=None, progress_note=None, review_copy_as_normal=False):
-	"""This function assumes that .generate_output() has already been called on each form."""
+	"""This function assumes that .generate_output() has already been called on each form.
+
+	It operates on the active patient.
+	"""
 
 	if len(forms) == 0:
 		return True
@@ -203,6 +206,8 @@ def act_on_generated_forms(parent=None, forms=None, jobtype=None, episode_name=N
 
 	#-----------------------------
 	def save_soap(soap=None):
+		if episode_name is None:
+			return
 		if soap.strip() == u'':
 			return
 		pat = gmPerson.gmCurrentPatient()
@@ -240,39 +245,6 @@ def act_on_generated_forms(parent=None, forms=None, jobtype=None, episode_name=N
 
 		return True
 	#-----------------------------
-	def save_forms():
-		# anything to do ?
-		files2save = []
-		form_names = []
-		for form in forms:
-			files2save.extend(form.final_output_filenames)
-			form_names.append(u'%s (%s)' % (form.template['name_long'], form.template['external_version']))
-		if len(files2save) == 0:
-			return True
-		# get path
-		path = os.path.expanduser(os.path.join('~', 'gnumed'))
-		dlg = wx.DirDialog (
-			parent = parent,
-			message = _('Select directory in which to create patient directory ...'),
-			defaultPath = path,
-			style = wx.DD_DEFAULT_STYLE
-		)
-		result = dlg.ShowModal()
-		path = dlg.GetPath()
-		dlg.Destroy()
-		if result != wx.ID_OK:
-			return
-		# save forms
-		pat = gmPerson.gmCurrentPatient()
-		path = os.path.join(path, pat.dirname)
-		gmTools.mkdir(path)
-		_log.debug('form saving path: %s', path)
-		for form in forms:
-			for filename in form.final_output_filenames:
-				shutil.copy2(filename, path)
-		soap_lines.append(_('Saved to disk: %s') % u', '.join(form_names))
-		return True
-	#-----------------------------
 	def print_forms():
 		# anything to do ?
 		files2print = []
@@ -293,67 +265,9 @@ def act_on_generated_forms(parent=None, forms=None, jobtype=None, episode_name=N
 		soap_lines.append(_('Printed: %s') % u', '.join(form_names))
 		return True
 	#-----------------------------
-	def mail_forms():
-		# anything to do ?
-		files2mail = []
-		form_names = []
-		for form in forms:
-			files2mail.extend(form.final_output_filenames)
-			form_names.append(u'%s (%s)' % (form.template['name_long'], form.template['external_version']))
-		if len(files2mail) == 0:
-			return True
-		found, external_cmd = gmShellAPI.detect_external_binary(u'gm-mail_doc')
-		if not found:
-			return False
-		# send mail
-		cmd = u'%s %s' % (external_cmd, u' '.join(files2mail))
-		if os.name == 'nt':
-			blocking = True
-		else:
-			blocking = False
-		success = gmShellAPI.run_command_in_shell (
-			command = cmd,
-			blocking = blocking
-		)
-		if not success:
-			gmGuiHelpers.gm_show_error (
-				aMessage = _('Error mailing documents.'),
-				aTitle = _('Mailing documents')
-			)
-			return False
-		soap_lines.append(_('Mailed: %s') % u', '.join(form_names))
-		return True
-	#-----------------------------
-	def fax_forms(fax_number=None):
-		# anything to do ?
-		files2fax = []
-		form_names = []
-		for form in forms:
-			files2fax.extend(form.final_output_filenames)
-			form_names.append(u'%s (%s)' % (form.template['name_long'], form.template['external_version']))
-		if len(files2fax) == 0:
-			return True
-		found, external_cmd = gmShellAPI.detect_external_binary(u'gm-fax_doc')
-		if not found:
-			return False
-		# send fax
-		cmd = u'%s "%s" %s' % (external_cmd, fax_number, u' '.join(files2fax))
-		if os.name == 'nt':
-			blocking = True
-		else:
-			blocking = False
-		success = gmShellAPI.run_command_in_shell (
-			command = cmd,
-			blocking = blocking
-		)
-		if not success:
-			gmGuiHelpers.gm_show_error (
-				aMessage = _('Error faxing documents to\n\n  %s') % fax_number,
-				aTitle = _('Faxing documents')
-			)
-			return False
-		soap_lines.append(_('Faxed to %s: %s') % (fax_number, u', '.join(form_names)))
-		return True
+	def export_forms(remote_print=False):
+		pat = gmPerson.gmCurrentPatient()
+		return pat.export_area.add_forms(forms = forms, designation = gmTools.bool2subst(remote_print, u'print', None, None))
 	#-----------------------------
 
 	if parent is None:
@@ -379,60 +293,29 @@ def act_on_generated_forms(parent=None, forms=None, jobtype=None, episode_name=N
 
 	progress_note = dlg.progress_note
 	episode_name = dlg._PRW_episode.GetValue().strip()
-	do_save = dlg._CHBOX_save.GetValue()
-	do_print = dlg._CHBOX_print.GetValue()
-	do_mail = dlg._CHBOX_mail.GetValue()
-	fax_number = dlg._PRW_fax.GetValue().strip()
+	if episode_name == u'':
+		episode_name = None
+	also_export = dlg._CHBOX_export.GetValue()
 	dlg.Destroy()
 
-	if action_code == wx.ID_OK:
-		if episode_name != u'':
-			result = archive_forms(episode_name = episode_name)
-		if do_save:
-			result = save_forms()
-		if do_print:
-			result = print_forms()
-		if do_mail:
-			result = mail_forms()
-		if fax_number != u'':
-			result = fax_forms(fax_number = fax_number)
-		if progress_note != u'':
-			soap_lines.insert(0, progress_note)
-		if len(soap_lines) > 0:
-			save_soap(soap = u'\n'.join(soap_lines))
-		return result
-
 	success = False
-	keep_a_copy = False
-	if action_code == _ID_FORM_DISPOSAL_PRINT_NOW:
-		if episode_name != u'':
-			keep_a_copy = True
+	if action_code == _ID_FORM_DISPOSAL_PRINT:
 		success = print_forms()
+		if episode_name is not None:
+			archive_forms(episode_name = episode_name)
+		if also_export:
+			export_forms()
 
-	elif action_code == _ID_FORM_DISPOSAL_ARCHIVE_NOW:
-		if episode_name == u'':
-			episode_name = None
-		keep_a_copy = True
-		success = True
+	if action_code == _ID_FORM_DISPOSAL_REMOTE_PRINT:
+		success = export_forms(remote_print = True)
+		if episode_name is not None:
+			archive_forms(episode_name = episode_name)
 
-	elif action_code == _ID_FORM_DISPOSAL_SAVE_NOW:
-		if episode_name != u'':
-			keep_a_copy = True
-		success = save_forms()
+	elif action_code == _ID_FORM_DISPOSAL_ARCHIVE_ONLY:
+		success = archive_forms(episode_name = episode_name)
 
-	elif action_code == _ID_FORM_DISPOSAL_MAIL_NOW:
-		if episode_name != u'':
-			keep_a_copy = True
-		success = mail_forms()
-
-	elif action_code == _ID_FORM_DISPOSAL_FAX_NOW:
-		if episode_name != u'':
-			keep_a_copy = True
-		success = fax_forms(fax_number = fax_number)
-
-	elif action_code == _ID_FORM_DISPOSAL_TRAY_NOW:
-		# not implemented
-		success = False
+	elif action_code == _ID_FORM_DISPOSAL_EXPORT_ONLY:
+		success = export_forms()
 
 	if not success:
 		return False
@@ -442,9 +325,6 @@ def act_on_generated_forms(parent=None, forms=None, jobtype=None, episode_name=N
 	if len(soap_lines) > 0:
 		save_soap(soap = u'\n'.join(soap_lines))
 
-	if keep_a_copy:
-		archive_forms(episode_name = episode_name)
-
 	return True
 
 #============================================================
@@ -502,58 +382,32 @@ class cFormDisposalDlg(wxgFormDisposalDlg.wxgFormDisposalDlg):
 	#--------------------------------------------------------
 	def __init_ui(self):
 		self._LCTRL_forms.set_columns([_('Form')])
-
-		self.__mail_script_exists, path = gmShellAPI.detect_external_binary(binary = u'gm-mail_doc')
-		if not self.__mail_script_exists:
-			self._LBL_mail.Disable()
-			self._CHBOX_mail.SetLabel(_('<gm-mail_doc(.bat) not found>'))
-			self._CHBOX_mail.SetValue(False)
-			self._CHBOX_mail.Disable()
-			self._BTN_mail.Disable()
-
-		self.__fax_script_exists, path = gmShellAPI.detect_external_binary(binary = u'gm-fax_doc')
-		if not self.__fax_script_exists:
-			self._LBL_fax.Disable()
-			self._PRW_fax.SetText(_('<gm-fax_doc(.bat) not found>'), data = None)
-			self._PRW_fax.display_as_disabled(True)
-			self._PRW_fax.Disable()
-			self._BTN_fax.Disable()
-
-		self._LBL_tray.Disable()
-		self._CHBOX_tray.SetValue(False)
-		self._CHBOX_tray.Disable()
-		self._BTN_tray.Disable()
-
+		#self._CHBOX_export.SetValue(False)
 	#--------------------------------------------------------
 	# event handlers
 	#--------------------------------------------------------
 	def _on_print_button_pressed(self, event):
-		self.EndModal(_ID_FORM_DISPOSAL_PRINT_NOW)
+		self.EndModal(_ID_FORM_DISPOSAL_PRINT)
 	#--------------------------------------------------------
-	def _on_mail_button_pressed(self, event):
-		event.Skip()
-		if not self.__mail_script_exists:
-			return
-		self.EndModal(_ID_FORM_DISPOSAL_MAIL_NOW)
+	def _on_remote_print_button_pressed(self, event):
+		self.EndModal(_ID_FORM_DISPOSAL_REMOTE_PRINT)
 	#--------------------------------------------------------
-	def _on_fax_button_pressed(self, event):
-		event.Skip()
-		if not self.__fax_script_exists:
-			return
-		self.EndModal(_ID_FORM_DISPOSAL_FAX_NOW)
-	#--------------------------------------------------------
-	def _on_tray_button_pressed(self, event):
-		self.EndModal(_ID_FORM_DISPOSAL_TRAY_NOW)
+	def _on_export_button_pressed(self, event):
+		self.EndModal(_ID_FORM_DISPOSAL_EXPORT_ONLY)
 	#--------------------------------------------------------
 	def _on_archive_button_pressed(self, event):
-		self.EndModal(_ID_FORM_DISPOSAL_ARCHIVE_NOW)
-	#--------------------------------------------------------
-	def _on_save_button_pressed(self, event):
-		self.EndModal(_ID_FORM_DISPOSAL_SAVE_NOW)
+		self.EndModal(_ID_FORM_DISPOSAL_ARCHIVE_ONLY)
 	#--------------------------------------------------------
 	def _on_show_forms_button_pressed(self, event):
 		event.Skip()
+		if self._LCTRL_forms.ItemCount == 0:
+			return
 		forms2show = self._LCTRL_forms.get_selected_item_data()
+		if len(forms2show) == 0:
+			data = self._LCTRL_forms.get_item_data(item_idx = 0)
+			if data is None:
+				return
+			forms2show = [data]
 		if len(forms2show) == 0:
 			return
 		for form in forms2show:
@@ -563,9 +417,6 @@ class cFormDisposalDlg(wxgFormDisposalDlg.wxgFormDisposalDlg):
 	def _on_delete_forms_button_pressed(self, event):
 		print "Event handler '_on_delete_forms_button_pressed' not implemented!"
 		event.Skip()
-	#--------------------------------------------------------
-	def _on_ok_button_pressed(self, event):
-		self.EndModal(wx.ID_OK)
 
 #============================================================
 # form template management
@@ -835,7 +686,7 @@ class cFormTemplateEAPnl(wxgFormTemplateEditAreaPnl.wxgFormTemplateEditAreaPnl,
 			defaultDir = os.path.expanduser(os.path.join('~', 'gnumed')),
 			defaultFile = '',
 			wildcard = '|'.join(wildcards),
-			style = wx.OPEN | wx.HIDE_READONLY | wx.FILE_MUST_EXIST
+			style = wx.OPEN | wx.FILE_MUST_EXIST
 		)
 		result = dlg.ShowModal()
 		if result != wx.ID_CANCEL:
@@ -868,7 +719,7 @@ class cFormTemplateEAPnl(wxgFormTemplateEditAreaPnl.wxgFormTemplateEditAreaPnl,
 			defaultDir = os.path.expanduser(os.path.join('~', 'gnumed')),
 			defaultFile = '',
 			wildcard = '|'.join(wildcards),
-			style = wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT | wx.HIDE_READONLY
+			style = wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT
 		)
 		result = dlg.ShowModal()
 		if result != wx.ID_CANCEL:
@@ -878,6 +729,255 @@ class cFormTemplateEAPnl(wxgFormTemplateEditAreaPnl.wxgFormTemplateEditAreaPnl,
 
 		event.Skip()
 
+#============================================================
+from Gnumed.wxGladeWidgets import wxgReceiverSelectionDlg
+
+class cReceiverSelectionDlg(wxgReceiverSelectionDlg.wxgReceiverSelectionDlg):
+
+	def __init__(self, *args, **kwargs):
+		wxgReceiverSelectionDlg.wxgReceiverSelectionDlg.__init__(self, *args, **kwargs)
+		self.__patient = None
+		self.__init_ui()
+		self.__register_interests()
+
+	#------------------------------------------------------------
+	def __init_ui(self):
+		if self.__patient is None:
+			return
+
+		self._LCTRL_candidates.item_tooltip_callback = self._get_candidate_tooltip
+		self.__populate_candidates_list()
+
+		self._LCTRL_addresses.item_tooltip_callback = self._get_tooltip
+		self._LCTRL_addresses.activate_callback = self._on_address_activated_in_list
+		adrs = self.__patient.get_addresses()
+		self.__populate_address_list(addresses = adrs)
+
+		self._TCTRL_final_name.SetValue(self.__patient[u'description'].strip())
+
+		self.Layout()
+	#------------------------------------------------------------
+	def __register_interests(self):
+		self._PRW_other_address.add_callback_on_selection(self._on_address_selected)
+		self._PRW_org_unit.add_callback_on_set_focus(self._on_entering_org_unit)
+		self._PRW_org_unit.add_callback_on_selection(self._on_org_unit_selected)
+
+	#------------------------------------------------------------
+	def __populate_candidates_list(self):
+		list_items = [[_('Patient'), self.__patient[u'description_gender'].strip()]]
+		list_data = [(self.__patient[u'description'].strip(), self.__patient.get_addresses(), u'')]
+
+		candidate_type = _('Emergency contact')
+		if self.__patient[u'emergency_contact'] is not None:
+			name = self.__patient[u'emergency_contact'].strip()
+			list_items.append([candidate_type, name])
+			list_data.append((name, [], u''))
+		contact = self.__patient.emergency_contact_in_database
+		if contact is not None:
+			list_items.append([candidate_type, contact[u'description_gender']])
+			list_data.append((contact[u'description'].strip(), contact.get_addresses(), u''))
+
+		candidate_type = _('Primary doctor')
+		prov = self.__patient.primary_provider
+		if prov is not None:
+			ident = prov.identity
+			list_items.append([candidate_type, u'%s: %s' % (prov[u'short_alias'], ident['description_gender'])])
+			list_data.append((ident['description'].strip(), ident.get_addresses(), _(u'in-praxis primary provider')))
+
+		candidate_type = _(u'This praxis')
+		branches = gmPraxis.get_praxis_branches(order_by = u'branch')
+		for branch in branches:
+			adr = branch.address
+			if adr is None:
+				continue
+			list_items.append([candidate_type, u'%s @ %s' % (branch['branch'], branch['praxis'])])
+			list_data.append((u'%s @ %s' % (branch['branch'], branch['praxis']), [adr], branch.format()))
+		del branches
+
+		candidate_type = _('External care')
+		cares = gmExternalCare.get_external_care_items(pk_identity = self.__patient.ID)
+		for care in cares:
+			details = u'%s%s@%s (%s)' % (
+				gmTools.coalesce(care['provider'], u'', u'%s: '),
+				care['unit'],
+				care['organization'],
+				care['issue']
+			)
+			name = (u'%s%s' % (
+				gmTools.coalesce(care['provider'], u'', u'%s, '),
+				u'%s @ %s' % (care['unit'], care['organization'])
+			)).strip()
+			adr = care.org_unit.address
+			if adr is None:
+				addresses = []
+			else:
+				addresses = [adr]
+			list_items.append([candidate_type, details])
+			tt = u'\n'.join(care.format(with_health_issue = True, with_address = True, with_comms = True))
+			list_data.append((name, addresses, tt))
+		del cares
+
+		emr = self.__patient.emr
+
+		candidate_type = _('Hospital stay')
+		depts = emr.get_attended_hospitals_as_org_units()
+		for dept in depts:
+			adr = dept.address
+			if adr is None:
+				continue
+			list_items.append([candidate_type, u'%s @ %s' % (dept['unit'], dept['organization'])])
+			list_data.append((u'%s @ %s' % (dept['unit'], dept['organization']), [adr], u'\n'.join(dept.format())))
+		del depts
+
+		candidate_type = _('Procedure')
+		proc_locs = emr.get_procedure_locations_as_org_units()
+		for proc_loc in proc_locs:
+			adr = proc_loc.address
+			if adr is None:
+				continue
+			list_items.append([candidate_type, u'%s @ %s' % (proc_loc['unit'], proc_loc['organization'])])
+			list_data.append((u'%s @ %s' % (proc_loc['unit'], proc_loc['organization']), [adr], u'\n'.join(proc_loc.format())))
+		del proc_locs
+
+		candidate_type = _('Lab')
+		labs = emr.get_labs_as_org_units()
+		for lab in labs:
+			adr = lab.address
+			if adr is None:
+				continue
+			list_items.append([candidate_type, u'%s @ %s' % (lab['unit'], lab['organization'])])
+			list_data.append((u'%s @ %s' % (lab['unit'], lab['organization']), [adr], u'\n'.join(lab.format())))
+		del labs
+
+		candidate_type = _('Bill receiver')
+		bills = self.__patient.bills
+		adrs_seen = []
+		for bill in bills:
+			if bill['pk_receiver_address'] in adrs_seen:
+				continue
+			adr = bill.address
+			if adr is None:
+				continue
+			adrs_seen.append(bill['pk_receiver_address'])
+			details = u'%s%s' % (bill['invoice_id'], gmDateTime.pydt_strftime(dt = bill['close_date'], format = ' (%Y %b %d)', none_str = u''))
+			list_items.append([candidate_type, details])
+			list_data.append((u'', [adr], u'\n'.join(adr.format())))
+
+		self._LCTRL_candidates.set_columns([_(u'Receiver'), _(u'Details')])
+		self._LCTRL_candidates.set_string_items(list_items)
+		self._LCTRL_candidates.set_column_widths()
+		self._LCTRL_candidates.set_data(list_data)
+
+	#------------------------------------------------------------
+	def _get_candidate_tooltip(self, data):
+		if data is None:
+			return u''
+		name, addresses, tt = data
+		return tt
+	#------------------------------------------------------------
+	def __populate_address_list(self, addresses=None):
+		self._LCTRL_addresses.Enable()
+		self._LCTRL_addresses.set_columns([_(u'Type'), _(u'Address')])
+		list_items = []
+		for a in addresses:
+			try:
+				a_type = a[u'l10n_address_type']
+			except KeyError:
+				a_type = u''
+			list_items.append([a_type, a.format(single_line = True, verbose = False, show_type = False)])
+
+		self._LCTRL_addresses.set_string_items(list_items)
+		self._LCTRL_candidates.set_column_widths()
+		self._LCTRL_addresses.set_data(addresses)
+
+	#------------------------------------------------------------
+	def _get_tooltip(self, data):
+		return u'\n'.join(data.format(show_type = True))
+	#------------------------------------------------------------
+	def _on_address_activated_in_list(self, evt):
+		evt.Skip()
+		adr = self._LCTRL_addresses.get_selected_item_data(only_one = True)
+		if adr is None:
+			return
+		self._PRW_other_address.address = adr[u'pk_address']
+		self._LBL_address_details.SetLabel(u'\n'.join(adr.format()))
+	#------------------------------------------------------------
+	#------------------------------------------------------------
+	def _on_address_selected(self, address):
+		if address is None:
+			self._LBL_address_details.SetLabel(u'')
+			return
+		self._LBL_address_details.SetLabel(u'\n'.join(self._PRW_other_address.address.format()))
+		self.Layout()
+	#------------------------------------------------------------
+	def _on_entering_org_unit(self):
+		self._LCTRL_addresses.Disable()
+	#------------------------------------------------------------
+	def _on_org_unit_selected(self, unit):
+		if unit is None:
+			self._LCTRL_addresses.remove_items_safely(max_tries = 3)
+			self._PRW_other_address.address = None
+			self._LBL_address_details.SetLabel(u'')
+			return
+		unit = self._PRW_org_unit.GetData(as_instance = True)
+		adr = unit.address
+		if adr is None:
+			self._LCTRL_addresses.remove_items_safely(max_tries = 3)
+			self._PRW_other_address.address = None
+			self._LBL_address_details.SetLabel(u'')
+			return
+		self._TCTRL_final_name.SetValue(self._PRW_org_unit.GetValue().strip())
+		self._LCTRL_addresses.set_columns([_(u'Address')])
+		self._LCTRL_addresses.set_string_items([adr.format(single_line = True, verbose = False, show_type = False)])
+		self._LCTRL_addresses.set_data([adr])
+		self._PRW_other_address.address = adr[u'pk_address']
+		tmp = u'%s\n\n%s' % (
+			u'\n'.join(unit.format()),
+			u'\n'.join(adr.format())
+		)
+		self._LBL_address_details.SetLabel(tmp)
+		self.Layout()
+	#------------------------------------------------------------
+	#------------------------------------------------------------
+	def _on_candidate_selected(self, event):
+		event.Skip()
+		name, addresses, tt = self._LCTRL_candidates.get_selected_item_data(only_one = True)
+		self._TCTRL_final_name.SetValue(name.strip())
+		self.__populate_address_list(addresses = addresses)
+	#------------------------------------------------------------
+	def _on_manage_addresses_button_pressed(self, event):
+		event.Skip()
+		manage_addresses(parent = self)
+	#------------------------------------------------------------
+	def _on_manage_orgs_button_pressed(self, event):
+		event.Skip()
+		from Gnumed.wxpython.gmOrganizationWidgets import manage_orgs
+		manage_orgs(parent = self, no_parent = False)
+	#------------------------------------------------------------
+	def _on_ok_button_pressed(self, event):
+		if self._TCTRL_final_name.GetValue().strip() == u'':
+			return False
+		if self._PRW_other_address.address is None:
+			return False
+		event.Skip()
+		self.EndModal(wx.ID_OK)
+	#------------------------------------------------------------
+	def _set_patient(self, patient):
+		self.__patient = patient
+		self.__init_ui()
+
+	patient = property(lambda x:x, _set_patient)
+	#------------------------------------------------------------
+	def _get_name(self):
+		return self._TCTRL_final_name.GetValue().strip()
+
+	name = property(_get_name, lambda x:x)
+	#------------------------------------------------------------
+	def _get_address(self):
+		return self._PRW_other_address.address
+
+	address = property(_get_address, lambda x:x)
+
 #============================================================
 # main
 #------------------------------------------------------------
