@@ -11,6 +11,7 @@ __license__ = "GPL v2 or later (details at http://www.gnu.org)"
 import os
 import logging
 import sys
+import codecs
 
 
 import wx
@@ -283,6 +284,78 @@ class cMultilineTextEntryDlg(wxgMultilineTextEntryDlg.wxgMultilineTextEntryDlg):
 		if self.original_text is not None:
 			self._TCTRL_text.SetValue(self.original_text)
 
+# ========================================================================
+def clipboard2text():
+
+	if wx.TheClipboard.IsOpened():
+		return False
+
+	if not wx.TheClipboard.Open():
+		return False
+
+	data_obj = wx.TextDataObject()
+	got_it = wx.TheClipboard.GetData(data_obj)
+	if got_it:
+		txt = data_obj.Text
+		wx.TheClipboard.Close()
+		return txt
+
+	wx.TheClipboard.Close()
+	return None
+
+#-------------------------------------------------------------------------
+def clipboard2file():
+
+	if wx.TheClipboard.IsOpened():
+		return False
+
+	if not wx.TheClipboard.Open():
+		return False
+
+	data_obj = wx.TextDataObject()
+	got_it = wx.TheClipboard.GetData(data_obj)
+	if got_it:
+		fname = gmTools.get_unique_filename(prefix = u'gm-clipboard-', suffix = u'.txt')
+		target_file = codecs.open(fname, u'wb', u'utf8')
+		target_file.write(data_obj.Text)
+		target_file.close()
+		wx.TheClipboard.Close()
+		return fname
+
+	data_obj = wx.BitmapDataObject()
+	got_it = wx.TheClipboard.GetData(data_obj)
+	if got_it:
+		fname = gmTools.get_unique_filename(prefix = u'gm-clipboard-', suffix = u'.png')
+		bmp = data_obj.Bitmap.SaveFile(fname, wx.BITMAP_TYPE_PNG)
+		wx.TheClipboard.Close()
+		return fname
+
+	wx.TheClipboard.Close()
+	return None
+
+#-------------------------------------------------------------------------
+def file2clipboard(filename=None, announce_result=False):
+
+	if wx.TheClipboard.IsOpened():
+		return False
+
+	if not wx.TheClipboard.Open():
+		return False
+
+	f = codecs.open(filename, u'rU', u'utf8')
+	data_obj = wx.TextDataObject()
+	data_obj.SetText(f.read())
+	f.close()
+	wx.TheClipboard.SetData(data_obj)
+	wx.TheClipboard.Close()
+	if announce_result:
+		gm_show_info (
+			title = _('file2clipboard'),
+			info = _('The file [%s] has been copied into the clipboard.') % filename
+		)
+
+	return True
+
 # ========================================================================
 class cFileDropTarget(wx.FileDropTarget):
 	"""Generic file drop target class.
@@ -301,6 +374,7 @@ class cFileDropTarget(wx.FileDropTarget):
 	#-----------------------------------------------
 	def OnDropFiles(self, x, y, filenames):
 		self.target.add_filenames(filenames)
+
 # ========================================================================
 def file2scaled_image(filename=None, height=100):
 	img_data = None
@@ -325,6 +399,7 @@ def file2scaled_image(filename=None, height=100):
 		return None
 
 	return bitmap
+
 # ========================================================================
 def gm_show_error(aMessage=None, aTitle = None, error=None, title=None):
 
@@ -439,7 +514,19 @@ if __name__ == '__main__':
 
 		return True
 	#------------------------------------------------------------------
+	def test_clipboard():
+		app = wx.PyWidgetTester(size = (200, 50))
+		result = clipboard2file()
+		if result is False:
+			print "problem opening clipboard"
+			return
+		if result is None:
+			print "no data in clipboard"
+			return
+		print "file:", result
+	#------------------------------------------------------------------
 	#test_scale_img()
-	test_sql_logic_prw()
+	#test_sql_logic_prw()
+	test_clipboard()
 
 #======================================================================
