@@ -11,6 +11,7 @@ import os.path
 
 
 import wx
+import wx.stc
 
 
 if __name__ == '__main__':
@@ -25,7 +26,7 @@ from Gnumed.wxpython import gmListWidgets
 
 _log = logging.getLogger('gm.ui')
 
-_text_expansion_fillin_regex = r'\$\[.*\]\$'
+_text_expansion_fillin_regex = r'\$\[[^]]*\]\$'
 
 #============================================================
 class cKeywordExpansion_TextCtrlMixin():
@@ -368,7 +369,7 @@ class cTextExpansionEditAreaPnl(wxgTextExpansionEditAreaPnl.wxgTextExpansionEdit
 			parent = self,
 			message = _('Choose the file containing the data snippet'),
 			wildcard = '|'.join(wildcards),
-			style = wx.OPEN | wx.HIDE_READONLY | wx.FILE_MUST_EXIST
+			style = wx.OPEN | wx.FILE_MUST_EXIST
 		)
 		result = dlg.ShowModal()
 		if result != wx.ID_CANCEL:
