@@ -13,6 +13,7 @@ import types
 import logging
 import os
 import codecs
+import datetime
 
 
 import wx
@@ -54,6 +55,7 @@ from Gnumed.wxpython import gmDemographicsWidgets
 from Gnumed.wxpython import gmDocumentWidgets
 from Gnumed.wxpython import gmKeywordExpansionWidgets
 from Gnumed.wxpython import gmPraxisWidgets
+from Gnumed.wxpython import gmAddressWidgets
 
 
 _log = logging.getLogger('gm.scripting')
@@ -75,9 +77,12 @@ _injectable_placeholders = {
 __known_variant_placeholders = {
 	# generic:
 	u'free_text': u"""show a dialog for entering some free text:
-		args: <message> shown in input dialog, must not contain '//' or '::'""",
+		args: <message> shown in input dialog, must not contain '//' or '::',
+		will cache input per <message>""",
+
 	u'text_snippet': u"""a text snippet, taken from the keyword expansion mechanism:
 		args: <snippet name>//<template>""",
+
 	u'data_snippet': u"""a binary snippet, taken from the keyword expansion mechanism:
 		args: <snippet name>//<template>//<optional target mime type>//<optional target extension>
 		returns full path to an exported copy of the
@@ -86,14 +91,129 @@ __known_variant_placeholders = {
 		target mime type: a mime type into which to convert the image, no conversion if not given
 		target extension: target file name extension, derived from target mime type if not given
 	""",
-	u'tex_escape': u"args: string to escape",
+
+	u'tex_escape': u"args: string to escape, mostly obsolete now",
+
 	u'today': u"args: strftime format",
+
 	u'gender_mapper': u"""maps gender of patient to a string:
 		args: <value when person is male> // <is female> // <is other>
 		eg. 'male//female//other'
 		or: 'Lieber Patient//Liebe Patientin'""",
 	u'client_version': u"the version of the current client as a string (no 'v' in front)",
 
+	u'gen_adr_street': u"""part of a generic address, cached, selected from database:
+		args: optional template//optional selection message//optional cache ID
+		template: %s-style formatting template
+		message: text message shown in address selection list
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'gen_adr_number': u"""part of a generic address, cached, selected from database:
+		args: optional template//optional selection message//optional cache ID
+		template: %s-style formatting template
+		message: text message shown in address selection list
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'gen_adr_subunit': u"""part of a generic address, cached, selected from database:
+		args: optional template//optional selection message//optional cache ID
+		template: %s-style formatting template
+		message: text message shown in address selection list
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'gen_adr_location': u"""part of a generic address, cached, selected from database:
+		args: optional template//optional selection message//optional cache ID
+		template: %s-style formatting template
+		message: text message shown in address selection list
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'gen_adr_suburb': u"""part of a generic address, cached, selected from database:
+		args: optional template//optional selection message//optional cache ID
+		template: %s-style formatting template
+		message: text message shown in address selection list
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'gen_adr_postcode': u"""part of a generic address, cached, selected from database:
+		args: optional template//optional selection message//optional cache ID
+		template: %s-style formatting template
+		message: text message shown in address selection list
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'gen_adr_region': u"""part of a generic address, cached, selected from database:
+		args: optional template//optional selection message//optional cache ID
+		template: %s-style formatting template
+		message: text message shown in address selection list
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'gen_adr_country': u"""part of a generic address, cached, selected from database:
+		args: optional template//optional selection message//optional cache ID
+		template: %s-style formatting template
+		message: text message shown in address selection list
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+
+	u'receiver_name': u"""the receiver name, cached, selected from database:
+		receivers are presented for selection from people/addresses related
+		to the patient in some way or other,
+		args: optional template//optional cache ID
+		template: %s-style formatting template
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'receiver_street': u"""part of a receiver address, cached, selected from database:
+		receivers are presented for selection from people/addresses related
+		to the patient in some way or other,
+		args: optional template//optional cache ID
+		template: %s-style formatting template
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'receiver_number': u"""part of a receiver address, cached, selected from database:
+		receivers are presented for selection from people/addresses related
+		to the patient in some way or other,
+		args: optional template//optional cache ID
+		template: %s-style formatting template
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'receiver_subunit': u"""part of a receiver address, cached, selected from database:
+		receivers are presented for selection from people/addresses related
+		to the patient in some way or other,
+		args: optional template//optional cache ID
+		template: %s-style formatting template
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'receiver_location': u"""part of a receiver address, cached, selected from database:
+		receivers are presented for selection from people/addresses related
+		to the patient in some way or other,
+		args: optional template//optional cache ID
+		template: %s-style formatting template
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'receiver_suburb': u"""part of a receiver address, cached, selected from database:
+		receivers are presented for selection from people/addresses related
+		to the patient in some way or other,
+		args: optional template//optional cache ID
+		template: %s-style formatting template
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'receiver_postcode': u"""part of a receiver address, cached, selected from database:
+		receivers are presented for selection from people/addresses related
+		to the patient in some way or other,
+		args: optional template//optional cache ID
+		template: %s-style formatting template
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'receiver_region': u"""part of a receiver address, cached, selected from database:
+		receivers are presented for selection from people/addresses related
+		to the patient in some way or other,
+		args: optional template//optional cache ID
+		template: %s-style formatting template
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
+	u'receiver_country': u"""part of a receiver address, cached, selected from database:
+		receivers are presented for selection from people/addresses related
+		to the patient in some way or other,
+		args: optional template//optional cache ID
+		template: %s-style formatting template
+		cache ID: used to differentiate separate cached invocations of this placeholder
+	""",
 
 	# patient demographics:
 	u'name': u"args: template for name parts arrangement",
@@ -193,6 +313,9 @@ __known_variant_placeholders = {
 		args:	<template>//<date format>
 		template:	something %(field)s something else (do not include '//' or '::' itself in the template)""",
 
+	u'external_care': u"""External care entries:
+		args:	<template>
+		template:	something %(field)s something else (do not include '//' or '::' itself in the template)""",
 
 	# provider related:
 	u'current_provider': u"no arguments",
@@ -208,6 +331,8 @@ __known_variant_placeholders = {
 		select:			if this is present allow selection of the branch rather than using the current branch""",
 
 	u'praxis_address': u"args: <optional formatting template>",
+	u'praxis_comm': u"args: type//<optional formatting template>",
+	u'praxis_id': u"args: <type of ID>//<issuer of ID>//<optional formatting template>",
 
 
 	# billing related:
@@ -481,6 +606,22 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 
 		return u'\n'.join(lines)
 	#--------------------------------------------------------
+	def _get_variant_external_care(self, data=None):
+
+		from Gnumed.wxpython import gmExternalCareWidgets
+		external_cares = gmExternalCareWidgets.manage_external_care()
+
+		if external_cares is None:
+			return u''
+
+		if len(external_cares) == 0:
+			return u''
+
+		template = data
+		lines = [ template % ext.fields_as_dict(escape_style = self.__esc_style) for ext in external_cares ]
+
+		return u'\n'.join(lines)
+	#--------------------------------------------------------
 	def _get_variant_documents(self, data=None):
 
 		select = False
@@ -838,6 +979,128 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 	#--------------------------------------------------------
 	# address related placeholders
 	#--------------------------------------------------------
+	def __get_variant_gen_adr_part(self, data=u'?', part=None):
+
+		template = u'%s'
+		msg = _('Select the address you want to use !')
+		cache_id = u''
+		options = data.split('//', 4)
+		if len(options) > 0:
+			template = options[0]
+			if template.strip() == u'':
+				template = u'%s'
+		if len(options) > 1:
+			msg = options[1]
+		if len(options) > 2:
+			cache_id = options[2]
+
+		cache_key = 'generic_address::' + cache_id
+		try:
+			adr2use = self.__cache[cache_key]
+			_log.debug('cache hit (%s): [%s]', cache_key, adr2use)
+		except KeyError:
+			adr2use = None
+
+		if adr2use is None:
+			dlg = gmAddressWidgets.cAddressSelectionDlg(None, -1)
+			dlg.message = msg
+			choice = dlg.ShowModal()
+			adr2use = dlg.address
+			dlg.Destroy()
+			if choice == wx.ID_CANCEL:
+				return u''
+			self.__cache[cache_key] = adr2use
+
+		return template % self._escape(adr2use[part])
+	#--------------------------------------------------------
+	def _get_variant_gen_adr_street(self, data=u'?'):
+		return self.__get_variant_gen_adr_part(data = data, part = 'street')
+	#--------------------------------------------------------
+	def _get_variant_gen_adr_number(self, data=u'?'):
+		return self.__get_variant_gen_adr_part(data = data, part = 'number')
+	#--------------------------------------------------------
+	def _get_variant_gen_adr_subunit(self, data=u'?'):
+		return self.__get_variant_gen_adr_part(data = data, part = 'subunit')
+	#--------------------------------------------------------
+	def _get_variant_gen_adr_location(self, data=u'?'):
+		return self.__get_variant_gen_adr_part(data = data, part = 'urb')
+	#--------------------------------------------------------
+	def _get_variant_gen_adr_suburb(self, data=u'?'):
+		return self.__get_variant_gen_adr_part(data = data, part = 'suburb')
+	#--------------------------------------------------------
+	def _get_variant_gen_adr_postcode(self, data=u'?'):
+		return self.__get_variant_gen_adr_part(data = data, part = 'postcode')
+	#--------------------------------------------------------
+	def _get_variant_gen_adr_region(self, data=u'?'):
+		return self.__get_variant_gen_adr_part(data = data, part = 'l10n_state')
+	#--------------------------------------------------------
+	def _get_variant_gen_adr_country(self, data=u'?'):
+		return self.__get_variant_gen_adr_part(data = data, part = 'l10n_country')
+	#--------------------------------------------------------
+	def __get_variant_receiver_part(self, data=u'%s', part=None):
+
+		template = u'%s'
+		cache_id = u''
+		options = data.split('//', 3)
+		if len(options) > 0:
+			template = options[0]
+			if template.strip() == u'':
+				template = u'%s'
+		if len(options) > 1:
+			cache_id = options[1]
+
+		cache_key = 'receiver::' + cache_id
+		try:
+			name, adr = self.__cache[cache_key]
+			_log.debug('cache hit (%s): [%s:%s]', cache_key, name, adr)
+		except KeyError:
+			name = None
+			adr = None
+
+		if name is None:
+			from Gnumed.wxpython import gmFormWidgets
+			dlg = gmFormWidgets.cReceiverSelectionDlg(None, -1)
+			dlg.patient = self.pat
+			choice = dlg.ShowModal()
+			name = dlg.name
+			adr = dlg.address
+			dlg.Destroy()
+			if choice == wx.ID_CANCEL:
+				return u''
+			self.__cache[cache_key] = (name, adr)
+
+		if part == u'name':
+			return template % self._escape(name)
+
+		return template % self._escape(gmTools.coalesce(adr[part], u''))
+	#--------------------------------------------------------
+	def _get_variant_receiver_name(self, data=u'%s'):
+		return self.__get_variant_receiver_part(data = data, part = 'name')
+	#--------------------------------------------------------
+	def _get_variant_receiver_street(self, data=u'%s'):
+		return self.__get_variant_receiver_part(data = data, part = 'street')
+	#--------------------------------------------------------
+	def _get_variant_receiver_number(self, data=u'%s'):
+		return self.__get_variant_receiver_part(data = data, part = 'number')
+	#--------------------------------------------------------
+	def _get_variant_receiver_subunit(self, data=u'%s'):
+		return self.__get_variant_receiver_part(data = data, part = 'subunit')
+	#--------------------------------------------------------
+	def _get_variant_receiver_location(self, data=u'%s'):
+		return self.__get_variant_receiver_part(data = data, part = 'urb')
+	#--------------------------------------------------------
+	def _get_variant_receiver_suburb(self, data=u'%s'):
+		return self.__get_variant_receiver_part(data = data, part = 'suburb')
+	#--------------------------------------------------------
+	def _get_variant_receiver_postcode(self, data=u'%s'):
+		return self.__get_variant_receiver_part(data = data, part = 'postcode')
+	#--------------------------------------------------------
+	def _get_variant_receiver_region(self, data=u'%s'):
+		return self.__get_variant_receiver_part(data = data, part = 'l10n_state')
+	#--------------------------------------------------------
+	def _get_variant_receiver_country(self, data=u'%s'):
+		return self.__get_variant_receiver_part(data = data, part = 'l10n_country')
+	#--------------------------------------------------------
 	def _get_variant_patient_address(self, data=u''):
 
 		data_parts = data.split(u'//')
@@ -1055,6 +1318,33 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 
 		return template % comms[0].fields_as_dict(escape_style = self.__esc_style)
 	#--------------------------------------------------------
+	def _get_variant_praxis_id(self, data=None):
+		options = data.split(u'//')
+		id_type = options[0].strip()
+		if id_type == u'':
+			return self._escape(u'praxis external ID: type is missing')
+
+		if len(options) > 1:
+			issuer = options[1].strip()
+			if issuer == u'':
+				issue = None
+		else:
+			issuer = None
+
+		if len(options) > 2:
+			template = options[2]
+		else:
+			template = u'%(name)s: %(value)s (%(issuer)s)'
+
+		ids = gmPraxis.gmCurrentPraxisBranch().get_external_ids(id_type = id_type, issuer = issuer)
+		if len(ids) == 0:
+			if self.debug:
+				return template + u': ' + self._escape(_('no external ID [%s] by [%s]') % (id_type, issuer))
+			return u''
+
+		return template % self._escape_dict(the_dict = ids[0], none_string = u'')
+
+	#--------------------------------------------------------
 	# provider related placeholders
 	#--------------------------------------------------------
 	def _get_variant_current_provider(self, data=None):
@@ -1462,29 +1752,34 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 		else:
 			msg = data
 
-		dlg = gmGuiHelpers.cMultilineTextEntryDlg (
-			None,
-			-1,
-			title = _('Replacing <free_text> placeholder'),
-			msg = _('Below you can enter free text.\n\n [%s]') % msg
-		)
-		dlg.enable_user_formatting = True
-		decision = dlg.ShowModal()
-
-		if decision != wx.ID_SAVE:
+		cache_key = u'free_text::%s' % msg
+		try:
+			text = self.__cache[cache_key]
+		except KeyError:
+			dlg = gmGuiHelpers.cMultilineTextEntryDlg (
+				None,
+				-1,
+				title = _('Replacing <free_text> placeholder'),
+				msg = _('Below you can enter free text.\n\n [%s]') % msg
+			)
+			dlg.enable_user_formatting = True
+			decision = dlg.ShowModal()
+			text = dlg.value.strip()
+			is_user_formatted = dlg.is_user_formatted
 			dlg.Destroy()
-			if self.debug:
-				return self._escape(_('Text input cancelled by user.'))
-			return u''
 
-		text = dlg.value.strip()
-		if dlg.is_user_formatted:
-			dlg.Destroy()
-			return text
+			if decision != wx.ID_SAVE:
+				if self.debug:
+					text = _('Text input cancelled by user.')
+				else:
+					text = u''
 
-		dlg.Destroy()
+			if not is_user_formatted:
+				text = self._escape(text)
 
-		return self._escape(text)
+			self.__cache[cache_key] = text
+
+		return text
 	#--------------------------------------------------------
 	def _get_variant_bill(self, data=None):
 		try:
@@ -1534,6 +1829,46 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 		if self.__esc_func is None:
 			return text
 		return self.__esc_func(text)
+	#--------------------------------------------------------
+	def _escape_dict(self, the_dict=None, date_format='%Y %b %d  %H:%M', none_string=u'', bool_strings=None):
+		if bool_strings is None:
+			bools = {True: u'true', False: u'false'}
+		else:
+			bools = {True: bool_strings[0], False: bool_strings[1]}
+		data = {}
+		for field in the_dict.keys():
+			# FIXME: harden against BYTEA fields
+			#if type(self._payload[self._idx[field]]) == ...
+			#	data[field] = _('<%s bytes of binary data>') % len(self._payload[self._idx[field]])
+			#	continue
+			val = the_dict[field]
+			if val is None:
+				data[field] = none_string
+				continue
+			if isinstance(val, bool):
+				data[field] = bools[val]
+				continue
+			if isinstance(val, datetime.datetime):
+				data[field] = gmDateTime.pydt_strftime(val, format = date_format, encoding = 'utf8')
+				if self.__esc_style in [u'latex', u'tex']:
+					data[field] = gmTools.tex_escape_string(data[field])
+				elif self.__esc_style in [u'xetex', u'xelatex']:
+					data[field] = gmTools.xetex_escape_string(data[field])
+				continue
+			try:
+				data[field] = unicode(val, encoding = 'utf8', errors = 'replace')
+			except TypeError:
+				try:
+					data[field] = unicode(val)
+				except (UnicodeDecodeError, TypeError):
+					val = '%s' % str(val)
+					data[field] = val.decode('utf8', 'replace')
+			if self.__esc_style in [u'latex', u'tex']:
+				data[field] = gmTools.tex_escape_string(data[field])
+			elif self.__esc_style in [u'xetex', u'xelatex']:
+				data[field] = gmTools.xetex_escape_string(data[field])
+		return data
+
 #=====================================================================
 class cMacroPrimitives:
 	"""Functions a macro can legally use.
@@ -1978,6 +2313,7 @@ if __name__ == '__main__':
 			#u'current_provider',
 			#u'current_provider_external_id::Starfleet Serial Number//Star Fleet Central Staff Office::1234',
 			#u'current_provider_external_id::LANR//LÄK::1234'
+			#u'$<current_provider_external_id::KV-LANR//KV::1234>$'
 			#u'primary_praxis_provider_external_id::LANR//LÄK::1234'
 			#u'form_name_long::::1234',
 			#u'form_name_long::::5',
@@ -2006,7 +2342,18 @@ if __name__ == '__main__':
 			#u'$<reminders:://::>$'
 			#u'$<current_meds_for_rx::%(brand)s (%(contains)s): dispense %(amount2dispense)s ::>$'
 			#u'$<praxis::%(branch)s (%(praxis)s)::>$'
-			u'$<praxis_address::::120>$'
+			#u'$<praxis_address::::120>$'
+			#u'$<praxis_id::::120>$'
+			#u'$<gen_adr_street::Street = %s//Wählen Sie die Empfängeradresse !::120>$', u'$<gen_adr_location::Ort = %s::120>$', u'$<gen_adr_country::::120>$'
+
+			#u'$<receiver_name::%s::120>$',
+			u'$<receiver_street::%s//a::120>$',
+			u'$<receiver_number:: %s//a::120>$',
+			#u'$<receiver_subunit:: %s::120>$',
+			u'$<receiver_postcode::%s//b::120>$',
+			#u'$<receiver_location:: %s::120>$',
+			#u'$<receiver_country::, %s::120>$',
+			#u'$<external_care::%(issue)s: %(provider)s of %(unit)s@%(organization)s (%(comment)s)::1024>$'
 		]
 
 		handler = gmPlaceholderHandler()
@@ -2039,13 +2386,15 @@ if __name__ == '__main__':
 		show_placeholders()
 	#--------------------------------------------------------
 
+	app = wx.App()
+
 	#test_placeholders()
 	#test_new_variant_placeholders()
 	#test_scripting()
 	#test_placeholder_regex()
 	#test()
-	#test_placeholder()
-	test_show_phs()
+	test_placeholder()
+	#test_show_phs()
 
 #=====================================================================
 
