@@ -1,6 +1,5 @@
 """Widgets dealing with address/contact information."""
 #============================================================
-__version__ = "$Revision: 1.175 $"
 __author__ = "R.Terry, SJ Tan, I Haywood, Carlos Moro <cfmoro1976@yahoo.es>"
 __license__ = 'GPL v2 or later (details at http://www.gnu.org)'
 
@@ -139,18 +138,18 @@ class cPersonAddressesManagerPnl(gmListWidgets.cGenericListManagerPnl):
 		])
 	#--------------------------------------------------------
 	def _add_address(self):
-		ea = gmAddressWidgets.cAddressEditAreaPnl(self, -1)
+		ea = gmAddressWidgets.cAddressEAPnl(self, -1)
 		ea.address_holder = self.__identity
-		dlg = gmEditArea.cGenericEditAreaDlg(self, -1, edit_area = ea)
+		dlg = gmEditArea.cGenericEditAreaDlg2(self, -1, edit_area = ea)
 		dlg.SetTitle(_('Adding new address'))
 		if dlg.ShowModal() == wx.ID_OK:
 			return True
 		return False
 	#--------------------------------------------------------
 	def _edit_address(self, address):
-		ea = gmAddressWidgets.cAddressEditAreaPnl(self, -1, address = address)
+		ea = gmAddressWidgets.cAddressEAPnl(self, -1, address = address)
 		ea.address_holder = self.__identity
-		dlg = gmEditArea.cGenericEditAreaDlg(self, -1, edit_area = ea)
+		dlg = gmEditArea.cGenericEditAreaDlg2(self, -1, edit_area = ea)
 		dlg.SetTitle(_('Editing address'))
 		if dlg.ShowModal() == wx.ID_OK:
 			# did we add an entirely new address ?
