@@ -2,6 +2,7 @@
 """
 #================================================================
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
+__license__ = "GPL v2 or later"
 
 import sys
 import logging
@@ -19,7 +20,6 @@ from Gnumed.pycommon import gmTools
 from Gnumed.pycommon import gmDispatcher
 from Gnumed.pycommon import gmMatchProvider
 from Gnumed.pycommon import gmDateTime
-from Gnumed.pycommon import gmNetworkTools
 
 from Gnumed.business import gmPerson
 from Gnumed.business import gmStaff
@@ -32,8 +32,6 @@ from Gnumed.wxpython import gmPlugin
 from Gnumed.wxpython import gmRegetMixin
 from Gnumed.wxpython import gmPhraseWheel
 from Gnumed.wxpython import gmEditArea
-from Gnumed.wxpython import gmAuthWidgets
-from Gnumed.wxpython import gmDataPackWidgets
 from Gnumed.wxpython.gmPatSearchWidgets import set_active_patient
 from Gnumed.wxpython.gmVaccWidgets import manage_vaccinations
 
@@ -88,67 +86,7 @@ class cMessageTypePhraseWheel(gmPhraseWheel.cPhraseWheel):
 			value = val,
 			data = gmProviderInbox.create_inbox_item_type(message_type = val)
 		)
-#====================================================================
-def _display_clinical_reminders():
-	wx.CallAfter(__display_clinical_reminders)
 
-gmDispatcher.connect(signal = u'post_patient_selection', receiver = _display_clinical_reminders)
-
-def __display_clinical_reminders():
-	pat = gmPerson.gmCurrentPatient()
-	if not pat.connected:
-		return
-	for msg in pat.overdue_messages:
-		if msg['expiry_date'] is None:
-			exp = u''
-		else:
-			exp = _(' - expires %s') % gmDateTime.pydt_strftime (
-				msg['expiry_date'],
-				'%Y %b %d',
-				accuracy = gmDateTime.acc_days
-			)
-		txt = _(
-			'Due for %s (since %s%s):\n'
-			'%s'
-			'%s'
-			'\n'
-			'Patient: %s\n'
-			'Reminder by: %s'
-		) % (
-			gmDateTime.format_interval_medically(msg['interval_due']),
-			gmDateTime.pydt_strftime(msg['due_date'], '%Y %b %d', accuracy = gmDateTime.acc_days),
-			exp,
-			gmTools.coalesce(msg['comment'], u'', u'\n%s\n'),
-			gmTools.coalesce(msg['data'], u'', u'\n%s\n'),
-			pat['description_gender'],
-			msg['modified_by']
-		)
-		gmGuiHelpers.gm_show_warning (
-			aTitle = _('Clinical reminder'),
-			aMessage = txt
-		)
-	for hint in pat.dynamic_hints:
-		txt = u'%s\n\n%s\n\n          %s' % (
-			hint['title'],
-			gmTools.wrap(hint['hint'], width = 50, initial_indent = u' ', subsequent_indent = u' '),
-			hint['source']
-		)
-		dlg = gmGuiHelpers.c2ButtonQuestionDlg (
-			None,
-			-1,
-			caption = _('Clinical hint'),
-			question = txt,
-			button_defs = [
-				{'label': _('OK'), 'tooltip': _('OK'), 'default': True},
-				{'label': _('More info'), 'tooltip': _('Go to [%s]') % hint['url']}
-			]
-		)
-		button = dlg.ShowModal()
-		dlg.Destroy()
-		if button == wx.ID_NO:
-			gmNetworkTools.open_url_in_browser(hint['url'], autoraise = False)
-
-	return
 #====================================================================
 from Gnumed.wxGladeWidgets import wxgInboxMessageEAPnl
 
@@ -424,6 +362,7 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 		else:
 			self._PRW_receiver.Enable(True)
 			self._PRW_receiver.SetText(value = u'', data = None)
+
 #============================================================
 def edit_inbox_message(parent=None, message=None, single_entry=True):
 
@@ -519,6 +458,11 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 		#gmRegetMixin.cRegetOnPaintMixin.repopulate_ui(self)
 		return True
 	#--------------------------------------------------------
+	def filter_by_active_patient(self):
+		self._CHBOX_active_patient.SetValue(True)
+		self._TXT_inbox_item_comment.SetValue(u'')
+		self.__populate_inbox()
+	#--------------------------------------------------------
 	# internal helpers
 	#--------------------------------------------------------
 	def __register_interests(self):
@@ -533,7 +477,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 	def __init_ui(self):
 		self._LCTRL_provider_inbox.debug = 'provider inbox list'
 
-		self._LCTRL_provider_inbox.set_columns([u'', _('Sent'), _('Status'), _('Category - Type'), _('Message')])
+		self._LCTRL_provider_inbox.set_columns([u'', _('Generated'), _('Status'), _('Category - Type'), _('Message')])
 		self._LCTRL_provider_inbox.searchable_columns = [2, 3, 4]
 		self._LCTRL_provider_inbox.item_tooltip_callback = self._get_msg_tooltip
 
@@ -570,7 +514,13 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 			include_without_provider = False
 
 		# get which messages to show
-		if self._RBTN_all_messages.GetValue():
+		if self._RBTN_relevant_messages.GetValue():
+			_log.debug('loading relevant messages')
+			self.__msgs = self.provider.inbox.get_relevant_messages (
+				pk_patient = pk_patient,
+				include_without_provider = include_without_provider
+			)
+		elif self._RBTN_all_messages.GetValue():
 			_log.debug('loading all but expired messages')
 			self.__msgs = self.provider.inbox.get_messages (
 				pk_patient = pk_patient,
@@ -633,7 +583,13 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 
 		items = []
 		for m in self.__msgs:
-			item = [_indicator[m['importance']], gmDateTime.pydt_strftime(m['received_when'], '%Y-%m-%d')]
+			item = [_indicator[m['importance']]]
+			item.append(u'%s: %s%s%s' % (
+				gmDateTime.pydt_strftime(m['received_when'], '%Y-%m-%d'),
+				m['modified_by'],
+				gmTools.u_right_arrow,
+				gmTools.coalesce(m['provider'], _(u'all'))
+			))
 			if m['due_date'] is None:
 				item.append(u'')
 			else:
@@ -659,33 +615,27 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 	#--------------------------------------------------------
 	def _on_results_mod_db(self):
 		_log.debug('reviewed_test_results_mod_db')
-		wx.CallAfter(self.__on_message_inbox_mod_db)
+		self._on_message_inbox_mod_db()
 	#--------------------------------------------------------
 	def _on_identity_mod_db(self):
 		_log.debug('identity_mod_db')
-		wx.CallAfter(self.__on_message_inbox_mod_db)
+		self._on_message_inbox_mod_db()
 	#--------------------------------------------------------
 	def _on_doc_obj_review_mod_db(self):
 		_log.debug('doc_obj_review_mod_db')
-		wx.CallAfter(self.__on_message_inbox_mod_db)
+		self._on_message_inbox_mod_db()
 	#--------------------------------------------------------
 	def _on_doc_mod_db(self):
 		_log.debug('doc_mod_db')
-		wx.CallAfter(self.__on_message_inbox_mod_db)
+		self._on_message_inbox_mod_db()
 	#--------------------------------------------------------
 	def _on_message_inbox_mod_db(self, *args, **kwargs):
 		_log.debug('message_inbox_mod_db')
-		wx.CallAfter(self.__on_message_inbox_mod_db)
-
-	def __on_message_inbox_mod_db(self):
 		self._schedule_data_reget()
 		gmDispatcher.send(signal = u'request_user_attention', msg = _('Please check your GNUmed Inbox !'))
 	#--------------------------------------------------------
 	def _on_post_patient_selection(self):
 		_log.debug('post_patient_selection')
-		wx.CallAfter(self.__on_post_patient_selection)
-
-	def __on_post_patient_selection(self):
 		self._CHBOX_active_patient.Enable()
 		self._schedule_data_reget()
 	#--------------------------------------------------------
@@ -810,9 +760,18 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 			gmDispatcher.send(signal = 'statustext', msg = _('You must deal with the reason for this message to remove it from your inbox.'), beep = True)
 			return False
 
+		# if not "to" current provider, then don't delete
+		if self.__focussed_msg['pk_staff'] != gmStaff.gmCurrentProvider()['pk_staff']:
+			gmDispatcher.send(signal = 'statustext', msg = _('This message can only be deleted by [%s].') % self.__focussed_msg['provider'], beep = True)
+			return False
+
 		pk_patient = self.__focussed_msg['pk_patient']
 		if pk_patient is not None:
-			emr = gmClinicalRecord.cClinicalRecord(aPKey = pk_patient, allow_user_interaction = False)
+			#emr = gmClinicalRecord.cClinicalRecord(aPKey = pk_patient, allow_user_interaction = False)
+			from Gnumed.wxpython import gmChartPullingWidgets
+			emr = gmChartPullingWidgets.pull_chart(gmPerson.cIdentity(pk_patient))
+			if emr is None:
+				return False
 			epi = emr.add_episode(episode_name = 'administrative', is_open = False)
 			soap_cat = gmTools.bool2subst (
 				(self.__focussed_msg['category'] == u'clinical'),
@@ -826,6 +785,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 		if not self.provider.inbox.delete_message(self.__focussed_msg['pk_inbox_message']):
 			gmDispatcher.send(signal='statustext', msg=_('Problem removing message from Inbox.'))
 			return False
+
 		return True
 	#--------------------------------------------------------
 	def _on_edit_focussed_msg(self, evt):
@@ -897,7 +857,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 			gmGuiHelpers.gm_show_error(msg, _('handling provider inbox item'))
 			return False
 
-		wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmShowMedDocs', sort_mode = 'review')
+		gmDispatcher.send(signal = 'display_widget', name = 'gmShowMedDocs', sort_mode = 'review')
 		return True
 	#--------------------------------------------------------
 	def _goto_measurements_review(self, pk_context=None, pk_patient=None):
@@ -925,7 +885,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 			gmGuiHelpers.gm_show_error(msg, _('handling provider inbox item'))
 			return False
 
-		wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmMeasurementsGridPlugin')
+		gmDispatcher.send(signal = 'display_widget', name = 'gmMeasurementsGridPlugin')
 		return True
 	#--------------------------------------------------------
 	def _goto_vaccination_review(self, pk_context=None, pk_patient=None):
@@ -956,61 +916,6 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 		wx.CallAfter(gmVaccWidgets.manage_vaccinations)
 
 		return True
-#============================================================
-def browse_dynamic_hints(parent=None):
-
-	if parent is None:
-		parent = wx.GetApp().GetTopWindow()
-	#------------------------------------------------------------
-	def get_tooltip(item):
-		if item is None:
-			return None
-		return item.format()
-	#------------------------------------------------------------
-	def switch_activation(item):
-		conn = gmAuthWidgets.get_dbowner_connection(procedure = _('Switching clinical hint activation'))
-		if conn is None:
-			return False
-		item['is_active'] = not item['is_active']
-		return item.save(conn = conn)
-	#------------------------------------------------------------
-	def manage_data_packs(item):
-		gmDataPackWidgets.manage_data_packs(parent = parent)
-		return True
-	#------------------------------------------------------------
-	def refresh(lctrl):
-		hints = gmProviderInbox.get_dynamic_hints(order_by = u'is_active DESC, source, hint')
-		items = [ [
-			gmTools.bool2subst(h['is_active'], gmTools.u_checkmark_thin, u''),
-			h['title'],
-			h['source'][:30],
-			h['hint'][:60],
-			gmTools.coalesce(h['url'], u'')[:60],
-			h['lang'],
-			h['pk']
-		] for h in hints ]
-		lctrl.set_string_items(items)
-		lctrl.set_data(hints)
-	#------------------------------------------------------------
-	gmListWidgets.get_choices_from_list (
-		parent = parent,
-		msg = _('\nDynamic hints registered with GNUmed.\n'),
-		caption = _('Showing dynamic hints.'),
-		columns = [ _('Active'), _('Title'), _('Source'), _('Hint'), u'URL', _('Language'), u'#' ],
-		single_selection = True,
-		refresh_callback = refresh,
-		left_extra_button = (
-			_('(De)-Activate'),
-			_('Switch activation of the selected hint'),
-			switch_activation
-		),
-		right_extra_button = (
-			_('Data packs'),
-			_('Browse and install clinical hints data packs'),
-			manage_data_packs
-		),
-		list_tooltip_callback = get_tooltip
-	)
 
 #============================================================
 if __name__ == '__main__':
