@@ -71,13 +71,10 @@ class cTopPnl(wxgTopPnl.wxgTopPnl):
 
 		# client internal signals
 		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
-		gmDispatcher.connect(signal = u'clin.allgergy_mod_db', receiver = self._on_allergies_change)
-		gmDispatcher.connect(signal = u'clin.allergy_state_mod_db', receiver = self._on_allergies_change)
-		gmDispatcher.connect(signal = u'dem.names_mod_db', receiver = self._on_name_identity_change)
-		gmDispatcher.connect(signal = u'dem.identity_mod_db', receiver = self._on_name_identity_change)
-		gmDispatcher.connect(signal = u'dem.identity_tag_mod_db', receiver = self._on_tag_change)
-
 		gmDispatcher.connect(signal = u'focus_patient_search', receiver = self._on_focus_patient_search)
+
+		gmDispatcher.connect(signal = u'gm_table_mod', receiver = self._on_database_signal)
+
 	#----------------------------------------------
 	# event handling
 	#----------------------------------------------
@@ -89,19 +86,47 @@ class cTopPnl(wxgTopPnl.wxgTopPnl):
 		dlg.ShowModal()
 		return
 	#----------------------------------------------
-	def _on_tag_change(self):
-		self.__update_tags()
-	#----------------------------------------------
-	def _on_name_identity_change(self):
-		self.__update_age_label()
+	def _on_database_signal(self, **kwds):
+
+		if kwds['table'] not in [u'dem.identity', u'dem.names', u'dem.identity_tag', u'clin.allergy', u'clin.allergy_state']:
+			return True
+
+		if self.curr_pat.connected:
+			# signal is not about our patient: ignore signal
+			if int(kwds['pk_identity']) != self.curr_pat.ID:
+				return True
+
+		if kwds['table'] == u'dem.identity':
+			# we don't care about newly INSERTed or DELETEd patients
+			if kwds['operation'] != 'UPDATE':
+				return True
+			self.__update_age_label()
+			return True
+
+		if kwds['table'] == u'dem.names':
+			self.__update_age_label()
+			return True
+
+		if kwds['table'] == u'dem.identity_tag':
+			self.__update_tags()
+			return True
+
+		if kwds['table'] == u'clin.allergy':
+			self.__update_allergies()
+			return True
+
+		if kwds['table'] == u'clin.allergy_state':
+			self.__update_allergies()
+			return True
+
+		return True
+
 	#----------------------------------------------
 	def _on_post_patient_selection(self, **kwargs):
 		self.__update_age_label()
 		self.__update_allergies()
 		self.__update_tags()
-	#-------------------------------------------------------
-	def _on_allergies_change(self, **kwargs):
-		self.__update_allergies()
+
 	#-------------------------------------------------------
 	def _on_focus_patient_search(self, **kwargs):
 		wx.CallAfter(self._TCTRL_patient_selector.SetFocus)
@@ -199,6 +224,13 @@ class cTopPnl(wxgTopPnl.wxgTopPnl):
 	#-------------------------------------------------------
 	def __update_allergies(self, **kwargs):
 
+		if not self.curr_pat.connected:
+			self._LBL_allergies.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOWTEXT))
+			self._TCTRL_allergies.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOWTEXT))
+			self._TCTRL_allergies.SetValue(u'')
+			self._TCTRL_allergies.SetToolTipString(u'')
+			return
+
 		show_red = True
 
 		emr = self.curr_pat.get_emr()
