@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #======================================================================
 # GNUmed patient EMR Journal plugin
 #======================================================================
@@ -19,7 +20,7 @@ class gmEMRJournalPlugin(gmPlugin.cNotebookPlugin):
 	"""Plugin to encapsulate patient EMR Journal window."""
 
 	tab_name = _('EMR journal')
-	required_minimum_role = 'doctor'
+	required_minimum_role = 'full clinical access'
 
 	def name (self):
 		return gmEMRJournalPlugin.tab_name
@@ -34,7 +35,6 @@ class gmEMRJournalPlugin(gmPlugin.cNotebookPlugin):
 		gmPlugin.cNotebookPlugin.register(self)
 	#-------------------------------------------------
 	def GetWidget (self, parent):
-		#self._widget = gmEMRBrowser.cEMRJournalPanel(parent, -1)
 		self._widget = gmEMRBrowser.cEMRJournalPluginPnl(parent, -1)
 		return self._widget
 
@@ -71,7 +71,7 @@ if __name__ == "__main__":
 
         # display standalone browser
         application = wx.wxPyWidgetTester(size=(800,600))
-        emr_journal = gmEMRBrowser.cEMRJournalPanel(application.frame, -1)
+        emr_journal = gmEMRBrowser.cEMRJournalPluginPnl(application.frame, -1)
         emr_journal.refresh_journal()
 
         application.frame.Show(True)
